import json

import datasets


_CITATION = """\
@misc{dinan2019second,
      title={The Second Conversational Intelligence Challenge (ConvAI2)},
      author={Emily Dinan and Varvara Logacheva and Valentin Malykh and Alexander Miller and Kurt Shuster and Jack Urbanek and Douwe Kiela and Arthur Szlam and Iulian Serban and Ryan Lowe and Shrimai Prabhumoye and Alan W Black and Alexander Rudnicky and Jason Williams and Joelle Pineau and Mikhail Burtsev and Jason Weston},
      year={2019},
      eprint={1902.00098},
      archivePrefix={arXiv},
      primaryClass={cs.AI}
}
"""

_DESCRIPTION = """\
ConvAI is a dataset of human-to-bot conversations labelled for quality. \
This data can be used to train a metric for evaluating dialogue systems. \
Moreover, it can be used in the development of chatbots themselves: it contains the information \
on the quality of utterances and entire dialogues, that can guide a dialogue system in search of better answers.
"""

_URL = "https://github.com/DeepPavlov/convai/raw/master/2018/data/summer_wild_evaluation_dialogs.json"


class ConvAi_2(datasets.GeneratorBasedBuilder):
    """ConvAI: A Dataset of Topic-Oriented Human-to-Chatbot Dialogues"""

    VERSION = datasets.Version("1.0.0")
    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="conv_ai_2",
            version=datasets.Version("1.0.0"),
            description="Full training set",
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "dialog_id": datasets.Value("string"),
                    "dialog": [
                        {
                            "id": datasets.Value("int32"),
                            "sender": datasets.Value("string"),
                            "text": datasets.Value("string"),
                            "sender_class": datasets.Value("string"),
                        }
                    ],
                    "bot_profile": datasets.Sequence([datasets.Value("string")]),
                    "user_profile": datasets.Sequence([datasets.Value("string")]),
                    "eval_score": datasets.Value("int32"),
                    "profile_match": datasets.Value("int32"),
                }
            ),
            supervised_keys=None,
            homepage="https://github.com/DeepPavlov/convai/tree/master/2018",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        downloaded_file = dl_manager.download(_URL)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": downloaded_file},
            ),
        ]

    def _generate_examples(self, filepath):
        with open(filepath, encoding="utf-8") as f:
            examples = json.load(f)
            for example in examples:
                example["id"] = example["dialog_id"]
                if example["eval_score"] is None:
                    example["eval_score"] = -1  # missing eval_score is replaced with -1
                if example["profile_match"] == "":
                    example["profile_match"] = -1  # missing profile_match is replaced with -1
                yield example["id"], example



"""TODO(squad_v2): Add a description here."""


import json
import os

import datasets


_CITATION = """\
No clear citation guidelines from source:
https://aiforthai.in.th/corpus.php
SQuAD version:
https://github.com/PyThaiNLP/thaiqa_squad
"""

_DESCRIPTION = """\
`thaiqa_squad` is an open-domain, extractive question answering dataset (4,000 questions in `train` and 74 questions in `dev`) in
[SQuAD](https://rajpurkar.github.io/SQuAD-explorer/) format, originally created by [NECTEC](https://www.nectec.or.th/en/) from
Wikipedia articles and adapted to [SQuAD](https://rajpurkar.github.io/SQuAD-explorer/) format by [PyThaiNLP](https://github.com/PyThaiNLP/).
"""


class ThaiQaSquadConfig(datasets.BuilderConfig):
    def __init__(self, **kwargs):
        """BuilderConfig

        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(ThaiQaSquadConfig, self).__init__(**kwargs)


class ThaiqaSquad(datasets.GeneratorBasedBuilder):
    _DOWNLOAD_URL = "https://github.com/PyThaiNLP/thaiqa_squad/raw/main/data.zip"
    _TRAIN_FILE = "train.jsonl"
    _VAL_FILE = "dev.jsonl"

    BUILDER_CONFIGS = [
        ThaiQaSquadConfig(
            name="thaiqa_squad",
            version=datasets.Version("1.0.0"),
            description="`thaiqa_squad` is an open-domain, extractive question answering dataset (4,000 questions in `train` and 74 questions in `dev`) in [SQuAD](https://rajpurkar.github.io/SQuAD-explorer/) format",
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    "question_id": datasets.Value("int32"),
                    "article_id": datasets.Value("int32"),
                    "context": datasets.Value("string"),
                    "question": datasets.Value("string"),
                    "answers": datasets.features.Sequence(
                        {
                            "answer": datasets.Value("string"),
                            "answer_begin_position": datasets.Value("int32"),
                            "answer_end_position": datasets.Value("int32"),
                        }
                    ),
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="https://github.com/PyThaiNLP/thaiqa_squad",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        arch_path = dl_manager.download_and_extract(self._DOWNLOAD_URL)
        data_dir = os.path.join(arch_path, "data")
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": os.path.join(data_dir, self._TRAIN_FILE)},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": os.path.join(data_dir, self._VAL_FILE)},
            ),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        with open(filepath, encoding="utf-8") as f:
            for id_, row in enumerate(f):
                data = json.loads(row)
                if not isinstance(data["answer"], list):
                    answer = [data["answer"]]
                    answer_begin_position = [data["answer_begin_position"]]
                    answer_end_position = [data["answer_end_position"]]
                yield id_, {
                    "question_id": data["question_id"],
                    "article_id": data["article_id"],
                    "context": data["context"],
                    "question": data["question"],
                    "answers": {
                        "answer": answer,
                        "answer_begin_position": answer_begin_position,
                        "answer_end_position": answer_end_position,
                    },
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Mac-Morpho dataset"""

import re

import datasets


logger = datasets.logging.get_logger(__name__)


_CITATION = """
@article{fonseca2015evaluating,
  title={Evaluating word embeddings and a revised corpus for part-of-speech tagging in Portuguese},
  author={Fonseca, Erick R and Rosa, Joao Luis G and Aluisio, Sandra Maria},
  journal={Journal of the Brazilian Computer Society},
  volume={21},
  number={1},
  pages={2},
  year={2015},
  publisher={Springer}
}
"""

_DESCRIPTION = """
Mac-Morpho is a corpus of Brazilian Portuguese texts annotated with part-of-speech tags.
Its first version was released in 2003 [1], and since then, two revisions have been made in order
to improve the quality of the resource [2, 3].
The corpus is available for download split into train, development and test sections.
These are 76%, 4% and 20% of the corpus total, respectively (the reason for the unusual numbers
is that the corpus was first split into 80%/20% train/test, and then 5% of the train section was
set aside for development). This split was used in [3], and new POS tagging research with Mac-Morpho
is encouraged to follow it in order to make consistent comparisons possible.


[1] Aluísio, S., Pelizzoni, J., Marchi, A.R., de Oliveira, L., Manenti, R., Marquiafável, V. 2003.
An account of the challenge of tagging a reference corpus for brazilian portuguese.
In: Proceedings of the 6th International Conference on Computational Processing of the Portuguese Language. PROPOR 2003

[2] Fonseca, E.R., Rosa, J.L.G. 2013. Mac-morpho revisited: Towards robust part-of-speech.
In: Proceedings of the 9th Brazilian Symposium in Information and Human Language Technology – STIL

[3] Fonseca, E.R., Aluísio, Sandra Maria, Rosa, J.L.G. 2015.
Evaluating word embeddings and a revised corpus for part-of-speech tagging in Portuguese.
Journal of the Brazilian Computer Society.
"""

_HOMEPAGE = "http://www.nilc.icmc.usp.br/macmorpho/"

_LICENSE = "Creative Commons Attribution 4.0 International License"

_URL = "http://www.nilc.icmc.usp.br/macmorpho/macmorpho-v3.tgz"


class MacMorpho(datasets.GeneratorBasedBuilder):
    """Mac-Morpho dataset."""

    VERSION = datasets.Version("3.0.0")

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "pos_tags": datasets.Sequence(
                        datasets.features.ClassLabel(
                            names=[
                                "PREP+PROADJ",
                                "IN",
                                "PREP+PRO-KS",
                                "NPROP",
                                "PREP+PROSUB",
                                "KC",
                                "PROPESS",
                                "NUM",
                                "PROADJ",
                                "PREP+ART",
                                "KS",
                                "PRO-KS",
                                "ADJ",
                                "ADV-KS",
                                "N",
                                "PREP",
                                "PROSUB",
                                "PREP+PROPESS",
                                "PDEN",
                                "V",
                                "PREP+ADV",
                                "PCP",
                                "CUR",
                                "ADV",
                                "PU",
                                "ART",
                            ]
                        )
                    ),
                }
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        archive = dl_manager.download(_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": "macmorpho-train.txt",
                    "files": dl_manager.iter_archive(archive),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": "macmorpho-test.txt",
                    "files": dl_manager.iter_archive(archive),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": "macmorpho-dev.txt",
                    "files": dl_manager.iter_archive(archive),
                },
            ),
        ]

    def _generate_examples(self, filepath, files):
        """Yields examples."""
        for path, f in files:
            if path == filepath:
                id_ = 0

                for line in f:

                    line = line.decode("utf-8").rstrip()
                    chunks = re.split(r"\s+", line)

                    tokens = []
                    pos_tags = []
                    for chunk in chunks:
                        token, tag = chunk.rsplit("_", 1)
                        tokens.append(token)
                        pos_tags.append(tag)

                    yield id_, {
                        "id": str(id_),
                        "tokens": tokens,
                        "pos_tags": pos_tags,
                    }
                    id_ += 1
                break



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
# Copyright 2019 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Yelp Polarity Reviews dataset."""


import os

import datasets
from datasets.tasks import TextClassification


_DESCRIPTION = """\
Large Yelp Review Dataset.
This is a dataset for binary sentiment classification. \
We provide a set of 560,000 highly polar yelp reviews for training, and 38,000 for testing. \

ORIGIN
The Yelp reviews dataset consists of reviews from Yelp. It is extracted
from the Yelp Dataset Challenge 2015 data. For more information, please
refer to http://www.yelp.com/dataset_challenge

The Yelp reviews polarity dataset is constructed by
Xiang Zhang (xiang.zhang@nyu.edu) from the above dataset.
It is first used as a text classification benchmark in the following paper:
Xiang Zhang, Junbo Zhao, Yann LeCun. Character-level Convolutional Networks
for Text Classification. Advances in Neural Information Processing Systems 28
(NIPS 2015).


DESCRIPTION

The Yelp reviews polarity dataset is constructed by considering stars 1 and 2
negative, and 3 and 4 positive. For each polarity 280,000 training samples and
19,000 testing samples are take randomly. In total there are 560,000 trainig
samples and 38,000 testing samples. Negative polarity is class 1,
and positive class 2.

The files train.csv and test.csv contain all the training samples as
comma-sparated values. There are 2 columns in them, corresponding to class
index (1 and 2) and review text. The review texts are escaped using double
quotes ("), and any internal double quote is escaped by 2 double quotes ("").
New lines are escaped by a backslash followed with an "n" character,
that is "\n".
"""

_CITATION = """\
@article{zhangCharacterlevelConvolutionalNetworks2015,
  archivePrefix = {arXiv},
  eprinttype = {arxiv},
  eprint = {1509.01626},
  primaryClass = {cs},
  title = {Character-Level {{Convolutional Networks}} for {{Text Classification}}},
  abstract = {This article offers an empirical exploration on the use of character-level convolutional networks (ConvNets) for text classification. We constructed several large-scale datasets to show that character-level convolutional networks could achieve state-of-the-art or competitive results. Comparisons are offered against traditional models such as bag of words, n-grams and their TFIDF variants, and deep learning models such as word-based ConvNets and recurrent neural networks.},
  journal = {arXiv:1509.01626 [cs]},
  author = {Zhang, Xiang and Zhao, Junbo and LeCun, Yann},
  month = sep,
  year = {2015},
}

"""

_DOWNLOAD_URL = "https://s3.amazonaws.com/fast-ai-nlp/yelp_review_polarity_csv.tgz"


class YelpPolarityReviewsConfig(datasets.BuilderConfig):
    """BuilderConfig for YelpPolarityReviews."""

    def __init__(self, **kwargs):
        """BuilderConfig for YelpPolarityReviews.

        Args:

            **kwargs: keyword arguments forwarded to super.
        """
        super(YelpPolarityReviewsConfig, self).__init__(version=datasets.Version("1.0.0", ""), **kwargs)


class YelpPolarity(datasets.GeneratorBasedBuilder):
    """Yelp Polarity reviews dataset."""

    BUILDER_CONFIGS = [
        YelpPolarityReviewsConfig(
            name="plain_text",
            description="Plain text",
        )
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "label": datasets.features.ClassLabel(names=["1", "2"]),
                }
            ),
            supervised_keys=None,
            homepage="https://course.fast.ai/datasets",
            citation=_CITATION,
            task_templates=[TextClassification(text_column="text", label_column="label")],
        )

    def _vocab_text_gen(self, train_file):
        for _, ex in self._generate_examples(train_file):
            yield ex["text"]

    def _split_generators(self, dl_manager):
        arch_path = dl_manager.download_and_extract(_DOWNLOAD_URL)
        train_file = os.path.join(arch_path, "yelp_review_polarity_csv", "train.csv")
        test_file = os.path.join(arch_path, "yelp_review_polarity_csv", "test.csv")
        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": train_file}),
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepath": test_file}),
        ]

    def _generate_examples(self, filepath):
        """Generate Yelp examples."""
        with open(filepath, encoding="utf-8") as f:
            for line_id, line in enumerate(f):
                # The format of the line is:
                # "1", "The text of the review."
                yield line_id, {"text": line[5:-2].strip(), "label": line[1]}



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""PANL BPPT"""


import os

import datasets


logger = datasets.logging.get_logger(__name__)


_CITATION = """\
@inproceedings{id_panl_bppt,
  author    = {PAN Localization - BPPT},
  title     = {Parallel Text Corpora, English Indonesian},
  year      = {2009},
  url       = {http://digilib.bppt.go.id/sampul/p92-budiono.pdf},
}
"""

_DESCRIPTION = """\
Parallel Text Corpora for Multi-Domain Translation System created by BPPT (Indonesian Agency for the Assessment and
Application of Technology) for PAN Localization Project (A Regional Initiative to Develop Local Language Computing
Capacity in Asia). The dataset contains around 24K sentences divided in 4 difference topics (Economic, international,
Science and Technology and Sport).
"""

_HOMEPAGE = "http://digilib.bppt.go.id/sampul/p92-budiono.pdf"

_LICENSE = ""

_URLs = ["https://github.com/cahya-wirawan/indonesian-language-models/raw/master/data/BPPTIndToEngCorpusHalfM.zip"]


class IdPanlBpptConfig(datasets.BuilderConfig):
    """BuilderConfig for IdPanlBppt"""

    def __init__(self, src_tag=None, tgt_tag=None, topics=None, **kwargs):
        """BuilderConfig for IdPanlBppt.
        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(IdPanlBpptConfig, self).__init__(**kwargs)
        self.src_tag = src_tag
        self.tgt_tag = tgt_tag
        self.topics = topics


class IdPanlBppt(datasets.GeneratorBasedBuilder):
    VERSION = datasets.Version("1.0.0")

    BUILDER_CONFIGS = [
        IdPanlBpptConfig(
            name="id_panl_bppt",
            version=VERSION,
            description="IdPanlBppt dataset",
            src_tag="en",
            tgt_tag="id",
            topics=[
                {"name": "Economy", "words": "150K"},
                {"name": "International", "words": "150K"},
                {"name": "Science", "words": "100K"},
                {"name": "Sport", "words": "100K"},
            ],
        ),
    ]
    BUILDER_CONFIG_CLASS = IdPanlBpptConfig

    def _info(self):
        features = datasets.Features(
            {
                "id": datasets.Value("string"),
                "translation": datasets.features.Translation(languages=[self.config.src_tag, self.config.tgt_tag]),
                "topic": datasets.features.ClassLabel(names=[topic["name"] for topic in self.config.topics]),
            }
        )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        my_urls = _URLs[0]
        data_dir = dl_manager.download_and_extract(my_urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "data_dir": os.path.join(data_dir, "plain"),
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, data_dir, split):
        logger.info("⏳ Generating %s examples from = %s", split, data_dir)
        id = 0
        for topic in self.config.topics:
            src_path = f"PANL-BPPT-{topic['name'][:3].upper()}-{self.config.src_tag.upper()}-{topic['words']}w.txt"
            tgt_path = f"PANL-BPPT-{topic['name'][:3].upper()}-{self.config.tgt_tag.upper()}-{topic['words']}w.txt"
            with open(os.path.join(data_dir, src_path), encoding="utf-8") as f1, open(
                os.path.join(data_dir, tgt_path), encoding="utf-8"
            ) as f2:
                src = f1.read().split("\n")[:-1]
                tgt = f2.read().split("\n")[:-1]
                for idx, (s, t) in enumerate(zip(src, tgt)):
                    yield id, {
                        "id": str(id),
                        "translation": {self.config.src_tag: s, self.config.tgt_tag: t},
                        "topic": topic["name"],
                    }
                    id += 1



import os

import numpy as np

import datasets


logger = datasets.logging.get_logger(__name__)


_CITATION = """
@misc{karpukhin2020dense,
    title={Dense Passage Retrieval for Open-Domain Question Answering},
    author={Vladimir Karpukhin and Barlas Oğuz and Sewon Min and Patrick Lewis and Ledell Wu and Sergey Edunov and Danqi Chen and Wen-tau Yih},
    year={2020},
    eprint={2004.04906},
    archivePrefix={arXiv},
    primaryClass={cs.CL}
}
"""

_DESCRIPTION = """
This is the wikipedia split used to evaluate the Dense Passage Retrieval (DPR) model.
It contains 21M passages from wikipedia along with their DPR embeddings.
The wikipedia articles were split into multiple, disjoint text blocks of 100 words as passages.
"""

_LICENSE = """DPR is CC-BY-NC 4.0 licensed."""

_DATA_URL = "https://dl.fbaipublicfiles.com/dpr/wikipedia_split/psgs_w100.tsv.gz"

_NQ_VECTORS_URL = "https://dl.fbaipublicfiles.com/dpr/data/wiki_encoded/single/nq/wiki_passages_{i}"

_MULTISET_VECTORS_URL = "https://dl.fbaipublicfiles.com/rag/rag_multiset_embeddings/wiki_passages_{i}"

_INDEX_URL = "https://storage.googleapis.com/huggingface-nlp/datasets/wiki_dpr"


class WikiDprConfig(datasets.BuilderConfig):
    """BuilderConfig for WikiDprConfig."""

    def __init__(
        self,
        with_embeddings=True,
        with_index=True,
        wiki_split="psgs_w100",
        embeddings_name="nq",
        index_name="compressed",
        index_train_size=262144,
        dummy=False,
        **kwargs,
    ):
        """BuilderConfig for WikiSnippets.
        Args:
            with_embeddings (`bool`, defaults to `True`): Load the 768-dimensional embeddings from DPR.
            with_index (`bool`, defaults to `True`): Load the faiss index trained on the embeddings.
            wiki_split (`str`, defaults to `psgs_w100`): name of the splitting method of wiki articles.
            embeddings_name (`str`, defaults to `nq`): "nq" or "multiset", depending on which dataset DPR was trained on.
            index_name (`str`, defaults to `compressed`): "compressed" or "exact", the configuration of the faiss index to use.
            index_train_size (`int`, defaults to `262144`): Size of the subset to use to train the index, if it is trainable.
            dummy (`bool`, defaults to `False`): Dummy uses only 10 000 examples for testing purposes.
          **kwargs: keyword arguments forwarded to super.
        """
        self.with_embeddings = with_embeddings
        self.with_index = with_index and index_name != "no_index"
        self.wiki_split = wiki_split
        self.embeddings_name = embeddings_name
        self.index_name = index_name if with_index else "no_index"
        self.index_train_size = index_train_size
        self.dummy = dummy
        name = [self.wiki_split, self.embeddings_name, self.index_name]
        if not self.with_embeddings:
            name.append("no_embeddings")
        if self.dummy:
            name = ["dummy"] + name
            assert (
                self.index_name != "compressed" or not self.with_index
            ), "Please use `index_name='exact' for dummy wiki_dpr`"
        assert wiki_split == "psgs_w100"
        assert embeddings_name in ("nq", "multiset")
        assert index_name in ("compressed", "exact", "no_index")
        kwargs["name"] = ".".join(name)
        super(WikiDprConfig, self).__init__(**kwargs)

        prefix = f"{wiki_split}.{embeddings_name}."
        if self.index_name == "exact":
            self.index_file = prefix + "HNSW128_SQ8-IP-{split}.faiss"
        else:
            self.index_file = prefix + "IVF4096_HNSW128_PQ128-IP-{split}.faiss"
        if self.dummy:
            self.index_file = "dummy." + self.index_file


class WikiDpr(datasets.GeneratorBasedBuilder):
    BUILDER_CONFIG_CLASS = WikiDprConfig
    BUILDER_CONFIGS = [
        WikiDprConfig(
            embeddings_name=embeddings_name,
            with_embeddings=with_embeddings,
            index_name=index_name,
            version=datasets.Version("0.0.0"),
        )
        for with_embeddings in (True, False)
        for embeddings_name in ("nq", "multiset")
        for index_name in ("exact", "compressed", "no_index")
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "embeddings": datasets.Sequence(datasets.Value("float32")),
                }
            )
            if self.config.with_embeddings
            else datasets.Features(
                {"id": datasets.Value("string"), "text": datasets.Value("string"), "title": datasets.Value("string")}
            ),
            supervised_keys=None,
            homepage="https://github.com/facebookresearch/DPR",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        files_to_download = {"data_file": _DATA_URL}
        downloaded_files = dl_manager.download_and_extract(files_to_download)
        if self.config.with_embeddings:
            vectors_url = _NQ_VECTORS_URL if self.config.embeddings_name == "nq" else _MULTISET_VECTORS_URL
            if self.config.dummy:
                downloaded_files["vectors_files"] = dl_manager.download([vectors_url.format(i=0)])
            else:
                downloaded_files["vectors_files"] = dl_manager.download([vectors_url.format(i=i) for i in range(50)])
        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs=downloaded_files),
        ]

    def _generate_examples(self, data_file, vectors_files=None):
        vec_idx = 0
        vecs = []
        lines = open(data_file, "r", encoding="utf-8")
        next(lines)  # skip headers
        for i, line in enumerate(lines):
            if self.config.dummy and i == 10000:
                break
            if i == 21015300:
                break  # ignore the last 24 examples for which the embeddings are missing.
            id, text, title = line.strip().split("\t")
            text = text[1:-1]  # remove " symbol at the beginning and the end
            text = text.replace('""', '"')  # replace double quotes by simple quotes
            if self.config.with_embeddings:
                if vec_idx >= len(vecs):
                    if len(vectors_files) == 0:
                        logger.warning(f"Ran out of vector files at index {i}")
                        break
                    vecs = np.load(open(vectors_files.pop(0), "rb"), allow_pickle=True)
                    vec_idx = 0
                vec_id, vec = vecs[vec_idx]
                assert int(id) == int(vec_id), f"ID mismatch between lines {id} and vector {vec_id}"
                yield id, {"id": id, "text": text, "title": title, "embeddings": vec}
                vec_idx += 1
            else:
                yield id, {
                    "id": id,
                    "text": text,
                    "title": title,
                }

    def _post_processing_resources(self, split):
        if self.config.with_index:
            return {"embeddings_index": self.config.index_file.format(split=split)}
        else:
            return {}

    def _download_post_processing_resources(self, split, resource_name, dl_manager):
        if resource_name == "embeddings_index":
            try:
                downloaded_resources = dl_manager.download_and_extract(
                    {"embeddings_index": _INDEX_URL + "/" + self.config.index_file.format(split=split)}
                )
                return downloaded_resources["embeddings_index"]
            except (FileNotFoundError, ConnectionError):  # index doesn't exist
                pass

    def _post_process(self, dataset, resources_paths):
        if self.config.with_index:
            index_file = resources_paths["embeddings_index"]
            if os.path.exists(index_file):
                dataset.load_faiss_index("embeddings", index_file)
            else:
                if "embeddings" not in dataset.column_names:
                    raise ValueError("Couldn't build the index because there are no embeddings.")
                import faiss

                d = 768
                train_size = self.config.index_train_size
                logger.info("Building wiki_dpr faiss index")
                if self.config.index_name == "exact":
                    index = faiss.IndexHNSWSQ(d, faiss.ScalarQuantizer.QT_8bit, 128, faiss.METRIC_INNER_PRODUCT)
                    index.hnsw.efConstruction = 200
                    index.hnsw.efSearch = 128
                    dataset.add_faiss_index("embeddings", custom_index=index, train_size=train_size)
                else:
                    quantizer = faiss.IndexHNSWFlat(d, 128, faiss.METRIC_INNER_PRODUCT)
                    quantizer.hnsw.efConstruction = 200
                    quantizer.hnsw.efSearch = 128
                    ivf_index = faiss.IndexIVFPQ(quantizer, d, 4096, 128, 8, faiss.METRIC_INNER_PRODUCT)
                    ivf_index.nprobe = 64
                    ivf_index.own_fields = True
                    quantizer.this.disown()
                    dataset.add_faiss_index(
                        "embeddings",
                        train_size=train_size,
                        custom_index=ivf_index,
                    )
                logger.info("Saving wiki_dpr faiss index")
                dataset.save_faiss_index("embeddings", index_file)
        return dataset



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""PubMedQA: A Dataset for Biomedical Research Question Answering"""


import json

import datasets


_CITATION = """\
@inproceedings{jin2019pubmedqa,
  title={PubMedQA: A Dataset for Biomedical Research Question Answering},
  author={Jin, Qiao and Dhingra, Bhuwan and Liu, Zhengping and Cohen, William and Lu, Xinghua},
  booktitle={Proceedings of the 2019 Conference on Empirical Methods in Natural Language Processing and the 9th International Joint Conference on Natural Language Processing (EMNLP-IJCNLP)},
  pages={2567--2577},
  year={2019}
}
"""

_DESCRIPTION = """\
PubMedQA is a novel biomedical question answering (QA) dataset collected from PubMed abstracts.
The task of PubMedQA is to answer research questions with yes/no/maybe (e.g.: Do preoperative
statins reduce atrial fibrillation after coronary artery bypass grafting?) using the corresponding abstracts.
PubMedQA has 1k expert-annotated, 61.2k unlabeled and 211.3k artificially generated QA instances.
Each PubMedQA instance is composed of (1) a question which is either an existing research article
title or derived from one, (2) a context which is the corresponding abstract without its conclusion,
(3) a long answer, which is the conclusion of the abstract and, presumably, answers the research question,
and (4) a yes/no/maybe answer which summarizes the conclusion.
PubMedQA is the first QA dataset where reasoning over biomedical research texts, especially their
quantitative contents, is required to answer the questions.
"""


_HOMEPAGE = "https://pubmedqa.github.io/"

_LICENSE = """\
MIT License
Copyright (c) 2019 pubmedqa
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
# TODO: Add link to the official dataset URLs here
# The HuggingFace dataset library don't host the datasets but only point to the original files
# This can be an arbitrary nested dict/list of URLs (see below in `_split_generators` method)
_URLs = {
    "ori_pqal": "https://raw.githubusercontent.com/pubmedqa/pubmedqa/master/data/ori_pqal.json",
    "ori_pqau": "https://drive.google.com/uc?export=download&id=1RsGLINVce-0GsDkCLDuLZmoLuzfmoCuQ",
    "ori_pqaa": "https://drive.google.com/uc?export=download&id=15v1x6aQDlZymaHGP7cZJZZYFfeJt2NdS",
}


class PubMedQAConfig(datasets.BuilderConfig):
    """BuilderConfig for PubMedQA"""

    def __init__(self, **kwargs):
        """
        Args:
            **kwargs: keyword arguments forwarded to super.
        """
        super(PubMedQAConfig, self).__init__(version=datasets.Version("1.0.0", ""), **kwargs)


class PubmedQA(datasets.GeneratorBasedBuilder):
    """PubMedQA: A Dataset for Biomedical Research Question Answering"""

    VERSION = datasets.Version("1.0.0")
    BUILDER_CONFIGS = [
        PubMedQAConfig(
            name="pqa_labeled",
            description="labeled: Two annotators labeled 1k instances with yes/no/maybe to build PQA-L(abeled) for fine-tuning",
        ),
        PubMedQAConfig(
            name="pqa_unlabeled",
            description="Unlabeled: Instances with yes/no/maybe answerable questions to build PQA-U(nlabeled)",
        ),
        PubMedQAConfig(
            name="pqa_artificial",
            description="Used simple heuristic to collect many noisily-labeled instances to build PQA-A for pretraining",
        ),
    ]

    def _info(self):
        if self.config.name == "pqa_labeled":
            return datasets.DatasetInfo(
                description=_DESCRIPTION,
                features=datasets.Features(
                    {
                        "pubid": datasets.Value("int32"),
                        "question": datasets.Value("string"),
                        "context": datasets.features.Sequence(
                            {
                                "contexts": datasets.Value("string"),
                                "labels": datasets.Value("string"),
                                "meshes": datasets.Value("string"),
                                "reasoning_required_pred": datasets.Value("string"),
                                "reasoning_free_pred": datasets.Value("string"),
                            }
                        ),
                        "long_answer": datasets.Value("string"),
                        "final_decision": datasets.Value("string"),
                    }
                ),
                supervised_keys=None,
                homepage=_HOMEPAGE,
                license=_LICENSE,
                citation=_CITATION,
            )
        elif self.config.name == "pqa_unlabeled":
            return datasets.DatasetInfo(
                description=_DESCRIPTION,
                features=datasets.Features(
                    {
                        "pubid": datasets.Value("int32"),
                        "question": datasets.Value("string"),
                        "context": datasets.features.Sequence(
                            {
                                "contexts": datasets.Value("string"),
                                "labels": datasets.Value("string"),
                                "meshes": datasets.Value("string"),
                            }
                        ),
                        "long_answer": datasets.Value("string"),
                    }
                ),
                supervised_keys=None,
                homepage=_HOMEPAGE,
                license=_LICENSE,
                citation=_CITATION,
            )
        elif self.config.name == "pqa_artificial":
            return datasets.DatasetInfo(
                description=_DESCRIPTION,
                features=datasets.Features(
                    {
                        "pubid": datasets.Value("int32"),
                        "question": datasets.Value("string"),
                        "context": datasets.features.Sequence(
                            {
                                "contexts": datasets.Value("string"),
                                "labels": datasets.Value("string"),
                                "meshes": datasets.Value("string"),
                            }
                        ),
                        "long_answer": datasets.Value("string"),
                        "final_decision": datasets.Value("string"),
                    }
                ),
                supervised_keys=None,
                homepage=_HOMEPAGE,
                license=_LICENSE,
                citation=_CITATION,
            )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        downloaded_files = dl_manager.download_and_extract(_URLs)
        if self.config.name == "pqa_labeled":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN, gen_kwargs={"filepath": downloaded_files["ori_pqal"]}
                )
            ]
        elif self.config.name == "pqa_artificial":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN, gen_kwargs={"filepath": downloaded_files["ori_pqaa"]}
                )
            ]
        elif self.config.name == "pqa_unlabeled":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN, gen_kwargs={"filepath": downloaded_files["ori_pqau"]}
                )
            ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        with open(filepath, encoding="utf-8") as f:
            data = json.load(f)
            for id_, row in enumerate(data):
                if self.config.name == "pqa_artificial":
                    yield id_, {
                        "pubid": row,
                        "question": data[row]["QUESTION"],
                        "context": {
                            "contexts": data[row]["CONTEXTS"],
                            "labels": data[row]["LABELS"],
                            "meshes": data[row]["MESHES"],
                        },
                        "long_answer": data[row]["LONG_ANSWER"],
                        "final_decision": data[row]["final_decision"],
                    }
                elif self.config.name == "pqa_labeled":
                    yield id_, {
                        "pubid": row,
                        "question": data[row]["QUESTION"],
                        "context": {
                            "contexts": data[row]["CONTEXTS"],
                            "labels": data[row]["LABELS"],
                            "meshes": data[row]["MESHES"],
                            "reasoning_required_pred": data[row]["reasoning_required_pred"],
                            "reasoning_free_pred": data[row]["reasoning_free_pred"],
                        },
                        "long_answer": data[row]["LONG_ANSWER"],
                        "final_decision": data[row]["final_decision"],
                    }
                elif self.config.name == "pqa_unlabeled":
                    yield id_, {
                        "pubid": row,
                        "question": data[row]["QUESTION"],
                        "context": {
                            "contexts": data[row]["CONTEXTS"],
                            "labels": data[row]["LABELS"],
                            "meshes": data[row]["MESHES"],
                        },
                        "long_answer": data[row]["LONG_ANSWER"],
                    }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import glob
import os
import re
from xml.dom.minidom import parseString

import datasets
from datasets.tasks import TextClassification


# no BibTeX citation
_CITATION = ""

_DESCRIPTION = """\
The Muchocine reviews dataset contains 3,872 longform movie reviews in Spanish language,
each with a shorter summary review, and a rating on a 1-5 scale.
"""

_LICENSE = "CC-BY-2.1"

_URLs = {"default": "http://www.lsi.us.es/~fermin/corpusCine.zip"}


class Muchocine(datasets.GeneratorBasedBuilder):
    VERSION = datasets.Version("1.1.1")

    def _info(self):
        features = datasets.Features(
            {
                "review_body": datasets.Value("string"),
                "review_summary": datasets.Value("string"),
                "star_rating": datasets.ClassLabel(names=[str(i) for i in range(1, 6)]),
            }
        )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage="http://www.lsi.us.es/~fermin/index.php/Datasets",
            license=_LICENSE,
            citation=_CITATION,
            task_templates=[
                TextClassification(text_column="review_body", label_column="star_rating"),
                TextClassification(text_column="review_summary", label_column="star_rating"),
            ],
        )

    def _split_generators(self, dl_manager):
        my_urls = _URLs[self.config.name]
        data_dir = dl_manager.download_and_extract(my_urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepaths": sorted(glob.glob(os.path.join(data_dir, "corpusCriticasCine", "*.xml"))),
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepaths, split):
        for filepath in filepaths:
            with open(filepath, encoding="latin-1") as f:
                id = re.search(r"\d+\.xml", filepath)[0][:-4]

                txt = f.read()
                txt = txt.replace("&ldquo;", '"').replace("&rdquo;", '"').replace("&hellip;", "")
                txt = txt.replace("&lsquo;", '"').replace("&rsquo;", '"').replace("&prime;", "")
                txt = txt.replace("&agrave;", "à").replace("&ndash;", "-").replace("&egrave;", "è")
                txt = txt.replace("&ouml;", "ö").replace("&ccedil;", "ç").replace("&", "and")
                try:
                    doc = parseString(txt)
                except Exception as e:
                    # skip 6 malformed xml files, for example unescaped < and >
                    _ = e
                    continue

                btxt = ""
                review_bod = doc.getElementsByTagName("body")
                if len(review_bod) > 0:
                    for node in review_bod[0].childNodes:
                        if node.nodeType == node.TEXT_NODE:
                            btxt += node.data + " "

                rtxt = ""
                review_summ = doc.getElementsByTagName("summary")
                if len(review_summ) > 0:
                    for node in review_summ[0].childNodes:
                        if node.nodeType == node.TEXT_NODE:
                            rtxt += node.data + " "

                yield id, {
                    "review_body": btxt,
                    "review_summary": rtxt,
                    "star_rating": doc.documentElement.attributes["rank"].value,
                }



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Introduction to the CoNLL-2002 Shared Task: Language-Independent Named Entity Recognition"""

import datasets


logger = datasets.logging.get_logger(__name__)


_CITATION = """\
@inproceedings{tjong-kim-sang-2002-introduction,
    title = "Introduction to the {C}o{NLL}-2002 Shared Task: Language-Independent Named Entity Recognition",
    author = "Tjong Kim Sang, Erik F.",
    booktitle = "{COLING}-02: The 6th Conference on Natural Language Learning 2002 ({C}o{NLL}-2002)",
    year = "2002",
    url = "https://www.aclweb.org/anthology/W02-2024",
}
"""

_DESCRIPTION = """\
Named entities are phrases that contain the names of persons, organizations, locations, times and quantities.

Example:
[PER Wolff] , currently a journalist in [LOC Argentina] , played with [PER Del Bosque] in the final years of the seventies in [ORG Real Madrid] .

The shared task of CoNLL-2002 concerns language-independent named entity recognition.
We will concentrate on four types of named entities: persons, locations, organizations and names of miscellaneous entities that do not belong to the previous three groups.
The participants of the shared task will be offered training and test data for at least two languages.
They will use the data for developing a named-entity recognition system that includes a machine learning component.
Information sources other than the training data may be used in this shared task.
We are especially interested in methods that can use additional unannotated data for improving their performance (for example co-training).

The train/validation/test sets are available in Spanish and Dutch.

For more details see https://www.clips.uantwerpen.be/conll2002/ner/ and https://www.aclweb.org/anthology/W02-2024/
"""

_URL = "https://raw.githubusercontent.com/teropa/nlp/master/resources/corpora/conll2002/"
_ES_TRAINING_FILE = "esp.train"
_ES_DEV_FILE = "esp.testa"
_ES_TEST_FILE = "esp.testb"
_NL_TRAINING_FILE = "ned.train"
_NL_DEV_FILE = "ned.testa"
_NL_TEST_FILE = "ned.testb"


class Conll2002Config(datasets.BuilderConfig):
    """BuilderConfig for Conll2002"""

    def __init__(self, **kwargs):
        """BuilderConfig forConll2002.

        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(Conll2002Config, self).__init__(**kwargs)


class Conll2002(datasets.GeneratorBasedBuilder):
    """Conll2002 dataset."""

    BUILDER_CONFIGS = [
        Conll2002Config(name="es", version=datasets.Version("1.0.0"), description="Conll2002 Spanish dataset"),
        Conll2002Config(name="nl", version=datasets.Version("1.0.0"), description="Conll2002 Dutch dataset"),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "pos_tags": datasets.Sequence(
                        datasets.features.ClassLabel(
                            names=[
                                "AO",
                                "AQ",
                                "CC",
                                "CS",
                                "DA",
                                "DE",
                                "DD",
                                "DI",
                                "DN",
                                "DP",
                                "DT",
                                "Faa",
                                "Fat",
                                "Fc",
                                "Fd",
                                "Fe",
                                "Fg",
                                "Fh",
                                "Fia",
                                "Fit",
                                "Fp",
                                "Fpa",
                                "Fpt",
                                "Fs",
                                "Ft",
                                "Fx",
                                "Fz",
                                "I",
                                "NC",
                                "NP",
                                "P0",
                                "PD",
                                "PI",
                                "PN",
                                "PP",
                                "PR",
                                "PT",
                                "PX",
                                "RG",
                                "RN",
                                "SP",
                                "VAI",
                                "VAM",
                                "VAN",
                                "VAP",
                                "VAS",
                                "VMG",
                                "VMI",
                                "VMM",
                                "VMN",
                                "VMP",
                                "VMS",
                                "VSG",
                                "VSI",
                                "VSM",
                                "VSN",
                                "VSP",
                                "VSS",
                                "Y",
                                "Z",
                            ]
                        )
                        if self.config.name == "es"
                        else datasets.features.ClassLabel(
                            names=["Adj", "Adv", "Art", "Conj", "Int", "Misc", "N", "Num", "Prep", "Pron", "Punc", "V"]
                        )
                    ),
                    "ner_tags": datasets.Sequence(
                        datasets.features.ClassLabel(
                            names=[
                                "O",
                                "B-PER",
                                "I-PER",
                                "B-ORG",
                                "I-ORG",
                                "B-LOC",
                                "I-LOC",
                                "B-MISC",
                                "I-MISC",
                            ]
                        )
                    ),
                }
            ),
            supervised_keys=None,
            homepage="https://www.aclweb.org/anthology/W02-2024/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        urls_to_download = {
            "train": f"{_URL}{_ES_TRAINING_FILE if self.config.name == 'es' else _NL_TRAINING_FILE}",
            "dev": f"{_URL}{_ES_DEV_FILE if self.config.name == 'es' else _NL_DEV_FILE}",
            "test": f"{_URL}{_ES_TEST_FILE if self.config.name == 'es' else _NL_TEST_FILE}",
        }
        downloaded_files = dl_manager.download_and_extract(urls_to_download)

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": downloaded_files["train"]}),
            datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs={"filepath": downloaded_files["dev"]}),
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepath": downloaded_files["test"]}),
        ]

    def _generate_examples(self, filepath):
        logger.info("⏳ Generating examples from = %s", filepath)
        with open(filepath, encoding="utf-8") as f:
            guid = 0
            tokens = []
            pos_tags = []
            ner_tags = []
            for line in f:
                if line.startswith("-DOCSTART-") or line == "" or line == "\n":
                    if tokens:
                        yield guid, {
                            "id": str(guid),
                            "tokens": tokens,
                            "pos_tags": pos_tags,
                            "ner_tags": ner_tags,
                        }
                        guid += 1
                        tokens = []
                        pos_tags = []
                        ner_tags = []
                else:
                    # conll2002 tokens are space separated
                    splits = line.split(" ")
                    tokens.append(splits[0])
                    pos_tags.append(splits[1])
                    ner_tags.append(splits[2].rstrip())
            # last example
            yield guid, {
                "id": str(guid),
                "tokens": tokens,
                "pos_tags": pos_tags,
                "ner_tags": ner_tags,
            }



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
import os

import datasets


_DESCRIPTION = """\
UMC005 English-Urdu is a parallel corpus of texts in English and Urdu language with sentence alignments. The corpus can be used for experiments with statistical machine translation.

The texts come from four different sources:
- Quran
- Bible
- Penn Treebank (Wall Street Journal)
- Emille corpus

The authors provide the religious texts of Quran and Bible for direct download. Because of licensing reasons, Penn and Emille texts cannot be redistributed freely. However, if you already hold a license for the original corpora, we are able to provide scripts that will recreate our data on your disk. Our modifications include but are not limited to the following:

- Correction of Urdu translations and manual sentence alignment of the Emille texts.
- Manually corrected sentence alignment of the other corpora.
- Our data split (training-development-test) so that our published experiments can be reproduced.
- Tokenization (optional, but needed to reproduce our experiments).
- Normalization (optional) of e.g. European vs. Urdu numerals, European vs. Urdu punctuation, removal of Urdu diacritics.
"""
_HOMEPAGE_URL = "http://ufal.ms.mff.cuni.cz/umc/005-en-ur/"
_URL = "http://ufal.ms.mff.cuni.cz/umc/005-en-ur/download.php?f=umc005-corpus.zip"
_CITATION = """\
@unpublished{JaZeWordOrderIssues2011,
author      = {Bushra Jawaid and Daniel Zeman},
title       = {Word-Order Issues in {English}-to-{Urdu} Statistical Machine Translation},
year        = {2011},
journal     = {The Prague Bulletin of Mathematical Linguistics},
number      = {95},
institution = {Univerzita Karlova},
address     = {Praha, Czechia},
issn        = {0032-6585},
}
"""

_ALL = "all"
_VERSION = "1.0.0"
_SOURCES = ["bible", "quran"]
_SOURCES_FILEPATHS = {
    s: {
        "train": {"urdu": "train.ur", "english": "train.en"},
        "dev": {"urdu": "dev.ur", "english": "dev.en"},
        "test": {"urdu": "test.ur", "english": "test.en"},
    }
    for s in _SOURCES
}


class UM005Config(datasets.BuilderConfig):
    def __init__(self, *args, sources=None, **kwargs):
        super().__init__(*args, version=datasets.Version(_VERSION, ""), **kwargs)
        self.sources = sources

    @property
    def language_pair(self):
        return ("ur", "en")


class UM005(datasets.GeneratorBasedBuilder):
    BUILDER_CONFIGS = [
        UM005Config(name=source, sources=[source], description=f"Source: {source}.") for source in _SOURCES
    ] + [
        UM005Config(
            name=_ALL,
            sources=_SOURCES,
            description="All sources included: bible, quran",
        )
    ]
    BUILDER_CONFIG_CLASS = UM005Config
    DEFAULT_CONFIG_NAME = _ALL

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "translation": datasets.Translation(languages=self.config.language_pair),
                },
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE_URL,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        path = dl_manager.download_and_extract(_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"datapath": path, "datatype": "train"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"datapath": path, "datatype": "dev"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"datapath": path, "datatype": "test"},
            ),
        ]

    def _generate_examples(self, datapath, datatype):
        if datatype == "train":
            ur_file = "train.ur"
            en_file = "train.en"
        elif datatype == "dev":
            ur_file = "dev.ur"
            en_file = "dev.en"
        elif datatype == "test":
            ur_file = "test.ur"
            en_file = "test.en"
        else:
            raise Exception("Invalid dataype. Try one of: dev, train, test")

        for source in self.config.sources:
            urdu_path = os.path.join(datapath, source, ur_file)
            english_path = os.path.join(datapath, source, en_file)
            with open(urdu_path, encoding="utf-8") as u, open(english_path, encoding="utf-8") as e:
                for sentence_counter, (x, y) in enumerate(zip(u, e)):
                    x = x.strip()
                    y = y.strip()
                    result = (
                        sentence_counter,
                        {
                            "id": str(sentence_counter),
                            "translation": {"ur": x, "en": y},
                        },
                    )
                    sentence_counter += 1
                    yield result



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Snips built in intents (2016-12-built-in-intents) dataset."""


import json

import datasets
from datasets.tasks import TextClassification


_DESCRIPTION = """\
Snips' built in intents dataset was initially used to compare different voice assistants and released as a public dataset hosted at
https://github.com/sonos/nlu-benchmark 2016-12-built-in-intents. The dataset contains 328 utterances over 10 intent classes. The
related paper mentioned on the github page is https://arxiv.org/abs/1805.10190 and a related Medium post is
https://medium.com/snips-ai/benchmarking-natural-language-understanding-systems-d35be6ce568d .
"""

_CITATION = """\
@article{DBLP:journals/corr/abs-1805-10190,
  author    = {Alice Coucke and
               Alaa Saade and
               Adrien Ball and
               Th{\'{e}}odore Bluche and
               Alexandre Caulier and
               David Leroy and
               Cl{\'{e}}ment Doumouro and
               Thibault Gisselbrecht and
               Francesco Caltagirone and
               Thibaut Lavril and
               Ma{\"{e}}l Primet and
               Joseph Dureau},
  title     = {Snips Voice Platform: an embedded Spoken Language Understanding system
               for private-by-design voice interfaces},
  journal   = {CoRR},
  volume    = {abs/1805.10190},
  year      = {2018},
  url       = {http://arxiv.org/abs/1805.10190},
  archivePrefix = {arXiv},
  eprint    = {1805.10190},
  timestamp = {Mon, 13 Aug 2018 16:46:59 +0200},
  biburl    = {https://dblp.org/rec/journals/corr/abs-1805-10190.bib},
  bibsource = {dblp computer science bibliography, https://dblp.org}
}
"""

_DOWNLOAD_URL = (
    "https://raw.githubusercontent.com/sonos/nlu-benchmark/master/2016-12-built-in-intents/benchmark_data.json"
)


class SnipsBuiltInIntents(datasets.GeneratorBasedBuilder):
    """Snips built in intents (2016-12-built-in-intents) dataset."""

    def _info(self):
        # ToDo: Consider adding an alternate configuration for the entity slots. The default is to only return the intent labels.

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "label": datasets.features.ClassLabel(
                        names=[
                            "ComparePlaces",
                            "RequestRide",
                            "GetWeather",
                            "SearchPlace",
                            "GetPlaceDetails",
                            "ShareCurrentLocation",
                            "GetTrafficInformation",
                            "BookRestaurant",
                            "GetDirections",
                            "ShareETA",
                        ]
                    ),
                }
            ),
            homepage="https://github.com/sonos/nlu-benchmark/tree/master/2016-12-built-in-intents",
            citation=_CITATION,
            task_templates=[TextClassification(text_column="text", label_column="label")],
        )

    def _split_generators(self, dl_manager):
        # Note: The source dataset doesn't have a train-test split.
        # ToDo: Consider splitting the data into train-test sets and re-hosting.
        samples_path = dl_manager.download_and_extract(_DOWNLOAD_URL)
        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": samples_path}),
        ]

    def _generate_examples(self, filepath):
        """Snips built in intent examples."""
        num_examples = 0

        with open(filepath, encoding="utf-8") as file_obj:
            snips_dict = json.load(file_obj)
            domains = snips_dict["domains"]

            for domain_dict in domains:
                intents = domain_dict["intents"]

                for intent_dict in intents:
                    label = intent_dict["benchmark"]["Snips"]["original_intent_name"]
                    queries = intent_dict["queries"]

                    for query_dict in queries:
                        query_text = query_dict["text"]

                        yield num_examples, {"text": query_text, "label": label}
                        num_examples += 1  # Explicitly keep track of the number of examples.



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""HAREM dataset"""


import json
import unicodedata
from typing import List, Tuple

import datasets


logger = datasets.logging.get_logger(__name__)


_CITATION = """
@inproceedings{santos2006harem,
  title={Harem: An advanced ner evaluation contest for portuguese},
  author={Santos, Diana and Seco, Nuno and Cardoso, Nuno and Vilela, Rui},
  booktitle={quot; In Nicoletta Calzolari; Khalid Choukri; Aldo Gangemi; Bente Maegaard; Joseph Mariani; Jan Odjik; Daniel Tapias (ed) Proceedings of the 5 th International Conference on Language Resources and Evaluation (LREC'2006)(Genoa Italy 22-28 May 2006)},
  year={2006}
}
"""

_DESCRIPTION = """
The HAREM is a Portuguese language corpus commonly used for Named Entity Recognition tasks. It includes about 93k words, from 129 different texts,
from several genres, and language varieties. The split of this dataset version follows the division made by [1], where 7% HAREM
documents are the validation set and the miniHAREM corpus (with about 65k words) is the test set. There are two versions of the dataset set,
a version that has a total of 10 different named entity classes (Person, Organization, Location, Value, Date, Title, Thing, Event,
Abstraction, and Other) and a "selective" version with only 5 classes (Person, Organization, Location, Value, and Date).

It's important to note that the original version of the HAREM dataset has 2 levels of NER details, namely "Category" and "Sub-type".
The dataset version processed here ONLY USE the "Category" level of the original dataset.

[1] Souza, Fábio, Rodrigo Nogueira, and Roberto Lotufo. "BERTimbau: Pretrained BERT Models for Brazilian Portuguese." Brazilian Conference on Intelligent Systems. Springer, Cham, 2020.
"""

_HOMEPAGE = "https://www.linguateca.pt/primeiroHAREM/harem_coleccaodourada_en.html"

_LICENSE = ""

_URLs = {
    "default": {
        "train": "https://raw.githubusercontent.com/neuralmind-ai/portuguese-bert/master/ner_evaluation/data/FirstHAREM-total-train.json",
        "dev": "https://raw.githubusercontent.com/neuralmind-ai/portuguese-bert/master/ner_evaluation/data/FirstHAREM-total-dev.json",
        "test": "https://raw.githubusercontent.com/neuralmind-ai/portuguese-bert/master/ner_evaluation/data/MiniHAREM-total.json",
    },
    "selective": {
        "train": "https://raw.githubusercontent.com/neuralmind-ai/portuguese-bert/master/ner_evaluation/data/FirstHAREM-selective-train.json",
        "dev": "https://raw.githubusercontent.com/neuralmind-ai/portuguese-bert/master/ner_evaluation/data/FirstHAREM-selective-dev.json",
        "test": "https://raw.githubusercontent.com/neuralmind-ai/portuguese-bert/master/ner_evaluation/data/MiniHAREM-selective.json",
    },
}


# method extracted from https://github.com/huggingface/transformers/blob/master/src/transformers/tokenization_utils.py#L77-L89
def _is_punctuation(char):
    """Checks whether `char` is a punctuation character."""
    cp = ord(char)
    # We treat all non-letter/number ASCII as punctuation.
    # Characters such as "^", "$", and "`" are not in the Unicode
    # Punctuation class but we treat them as punctuation anyways, for
    # consistency.
    if (cp >= 33 and cp <= 47) or (cp >= 58 and cp <= 64) or (cp >= 91 and cp <= 96) or (cp >= 123 and cp <= 126):
        return True
    cat = unicodedata.category(char)
    if cat.startswith("P"):
        return True
    return False


# method extracted from https://github.com/huggingface/transformers/blob/master/src/transformers/tokenization_utils.py#L53-L62
def _is_whitespace(char):
    """Checks whether `char` is a whitespace character."""
    # \t, \n, and \r are technically control characters but we treat them
    # as whitespace since they are generally considered as such.
    if char == " " or char == "\t" or char == "\n" or char == "\r":
        return True
    cat = unicodedata.category(char)
    if cat == "Zs":
        return True
    return False


class Token:
    """Info about a single token."""

    def __init__(self, text: str, tail: str = ""):

        if not isinstance(text, str) or not text:
            raise TypeError("text should be a non-empty string.")
        self.text = text
        self.tail = tail

    def __len__(self):
        return len(self.text) + len(self.tail)

    def __add__(self, char):
        self.text += char
        return self


def reconstruct_text_from_tokens(tokens: List[Token], include_last_tail: bool = False) -> str:
    """Concatenates the text of a sequence of tokens."""

    def text_generator(tokens):
        for i, token in enumerate(tokens):
            yield token.text
            if i < len(tokens) - 1 or include_last_tail:
                yield token.tail

    return "".join(piece for piece in text_generator(tokens))


def tokenize(text: str) -> Tuple[List[Token], List[int]]:
    """Perform whitespace and punctuation tokenization keeping track of char alignment"""
    doc_tokens = []
    char_to_word_offset = []

    new_word = True
    curr_token = None

    def begin_new_token(doc_tokens, text):
        token = Token(text=text)
        doc_tokens.append(token)
        return token

    for offset, c in enumerate(text):
        if _is_whitespace(c):
            new_word = True
            if curr_token:
                curr_token.tail += c
        else:
            if _is_punctuation(c):
                curr_token = begin_new_token(doc_tokens, c)
                new_word = True
            else:
                if new_word:
                    curr_token = begin_new_token(doc_tokens, c)
                else:
                    curr_token += c
                new_word = False

        # OBS: Whitespaces that appear before any tokens will have offset -1
        # char_to_word_offset.append(len(doc_tokens) - 1)
        char_to_word_offset.append(max(0, len(doc_tokens) - 1))

    return doc_tokens, char_to_word_offset


class HAREM(datasets.GeneratorBasedBuilder):
    """HAREM dataset."""

    VERSION = datasets.Version("1.0.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="default",
            version=VERSION,
            description="All the tags (PESSOA, ORGANIZACAO, LOCAL, TEMPO, VALOR, ABSTRACCAO, ACONTECIMENTO, COISA, OBRA, OUTRO) will be used",
        ),
        datasets.BuilderConfig(
            name="selective",
            version=VERSION,
            description="Only a subset of the tags (PESSOA, ORGANIZACAO, LOCAL, TEMPO, VALOR) will be used",
        ),
    ]

    DEFAULT_CONFIG_NAME = "default"

    def _info(self):

        tags = [
            "O",
            "B-PESSOA",
            "I-PESSOA",
            "B-ORGANIZACAO",
            "I-ORGANIZACAO",
            "B-LOCAL",
            "I-LOCAL",
            "B-TEMPO",
            "I-TEMPO",
            "B-VALOR",
            "I-VALOR",
        ]

        if self.config.name == "default":
            tags += [
                "B-ABSTRACCAO",
                "I-ABSTRACCAO",
                "B-ACONTECIMENTO",
                "I-ACONTECIMENTO",
                "B-COISA",
                "I-COISA",
                "B-OBRA",
                "I-OBRA",
                "B-OUTRO",
                "I-OUTRO",
            ]

        features = datasets.Features(
            {
                "id": datasets.Value("string"),
                "tokens": datasets.Sequence(datasets.Value("string")),
                "ner_tags": datasets.Sequence(datasets.features.ClassLabel(names=tags)),
            }
        )

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        my_urls = _URLs[self.config.name]
        data_dir = dl_manager.download_and_extract(my_urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_dir["train"], "split": "train"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": data_dir["test"], "split": "test"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": data_dir["dev"], "split": "dev"},
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""

        logger.info("⏳ Generating examples from = %s", filepath)

        with open(filepath, "r", encoding="utf-8") as f:

            input_data = json.load(f)
            id_ = 0

            for document in input_data:
                doc_text = document["doc_text"]
                doc_id = document["doc_id"]

                doc_tokens, char_to_word_offset = tokenize(doc_text)
                tags = ["O"] * len(doc_tokens)

                def set_label(index, tag):
                    if tags[index] != "O":
                        logger.warning(
                            "Overwriting tag %s at position %s to %s",
                            tags[index],
                            index,
                            tag,
                        )
                    tags[index] = tag

                for entity in document["entities"]:
                    entity_text = entity["text"]
                    entity_type = entity["label"]
                    start_token = None
                    end_token = None

                    entity_start_offset = entity["start_offset"]
                    entity_end_offset = entity["end_offset"]
                    start_token = char_to_word_offset[entity_start_offset]

                    # end_offset is NOT inclusive to the text, e.g.,
                    # entity_text == doc_text[start_offset:end_offset]
                    end_token = char_to_word_offset[entity_end_offset - 1]

                    assert start_token <= end_token, "End token cannot come before start token."
                    reconstructed_text = reconstruct_text_from_tokens(doc_tokens[start_token : (end_token + 1)])
                    assert (
                        entity_text.strip() == reconstructed_text
                    ), "Entity text and reconstructed text are not equal: %s != %s" % (
                        entity_text,
                        reconstructed_text,
                    )

                    for token_index in range(start_token, end_token + 1):
                        if token_index == start_token:
                            tag = "B-" + entity_type
                        else:
                            tag = "I-" + entity_type
                        set_label(token_index, tag)

                yield id_, {
                    "id": doc_id,
                    "tokens": [x.text for x in doc_tokens],
                    "ner_tags": tags,
                }
                id_ += 1



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Librispeech language modeling dataset."""


import datasets


_CITATION = """\
@inproceedings{panayotov2015librispeech,
  title={Librispeech: an ASR corpus based on public domain audio books},
  author={Panayotov, Vassil and Chen, Guoguo and Povey, Daniel and Khudanpur, Sanjeev},
  booktitle={Acoustics, Speech and Signal Processing (ICASSP), 2015 IEEE International Conference on},
  pages={5206--5210},
  year={2015},
  organization={IEEE}
}
"""

_DESCRIPTION = """\
Language modeling resources to be used in conjunction with the LibriSpeech ASR corpus.
"""

_URL = "http://www.openslr.org/11"

_DL_URL = "http://www.openslr.org/resources/11/librispeech-lm-norm.txt.gz"


class LibrispeechLm(datasets.GeneratorBasedBuilder):
    """Librispeech language modeling dataset."""

    VERSION = datasets.Version("0.1.0")

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "text": datasets.Value("string"),
                }
            ),
            supervised_keys=("text", "text"),
            homepage=_URL,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        archive_path = dl_manager.download_and_extract(_DL_URL)
        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"archive_path": archive_path}),
        ]

    def _generate_examples(self, archive_path):
        """Yields examples."""
        with open(archive_path, "r", encoding="utf-8") as f:
            for key, line in enumerate(f):
                text = line.strip()
                if text:  # Skip empty lines.
                    yield key, {"text": text}



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""TODO: Add a description here."""


import json

import datasets


_CITATION = """\
@inproceedings{swapnil2020,
    title={An Annotated Dataset of Discourse Modes in Hindi Stories},
    author={Swapnil Dhanwal, Hritwik Dutta, Hitesh Nankani, Nilay Shrivastava, Yaman Kumar, Junyi Jessy Li, Debanjan Mahata, Rakesh Gosangi, Haimin Zhang, Rajiv Ratn Shah, Amanda Stent},
    booktitle={Proceedings of the 12th Conference on Language Resources and Evaluation (LREC 2020)},
    volume={12},
    pages={1191–1196},
    year={2020}
"""

_DESCRIPTION = """\
The Hindi Discourse Analysis dataset is a corpus for analyzing discourse modes present in its sentences.
It contains sentences from stories written by 11 famous authors from the 20th Century.
4-5 stories by each author have been selected which were available in the public domain resulting
in a collection of 53 stories. Most of these short stories were originally written in Hindi
but some of them were written in other Indian languages and later translated to Hindi.
"""


_DOWNLOAD_URL = "https://raw.githubusercontent.com/midas-research/hindi-discourse/master/discourse_dataset.json"


class HindiDiscourse(datasets.GeneratorBasedBuilder):
    """Hindi Discourse Dataset - dataset of discourse modes in Hindi stories."""

    VERSION = datasets.Version("1.0.0")

    def _info(self):
        # This method pecifies the datasets.DatasetInfo object which contains informations and typings for the dataset

        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "Story_no": datasets.Value("int32"),
                    "Sentence": datasets.Value("string"),
                    "Discourse Mode": datasets.ClassLabel(
                        names=["Argumentative", "Descriptive", "Dialogue", "Informative", "Narrative", "Other"]
                    ),
                }
            ),
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="https://github.com/midas-research/hindi-discourse",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        dataset_path = dl_manager.download_and_extract(_DOWNLOAD_URL)

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": dataset_path}),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        with open(filepath, encoding="utf-8") as f:
            hindiDiscourse = json.load(f)

            for sentence, rowData in hindiDiscourse.items():
                yield sentence, {
                    "Story_no": rowData["Story_no"],
                    "Sentence": rowData["Sentence"],
                    "Discourse Mode": rowData["Discourse Mode"],
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# Lint as: python3
"""TTC4900: A Benchmark Data for Turkish Text Categorization"""


import csv

import datasets
from datasets.tasks import TextClassification


logger = datasets.logging.get_logger(__name__)


_DESCRIPTION = """\
The data set is taken from kemik group
http://www.kemik.yildiz.edu.tr/
The data are pre-processed for the text categorization, collocations are found, character set is corrected, and so forth.
We named TTC4900 by mimicking the name convention of TTC 3600 dataset shared by the study http://journals.sagepub.com/doi/abs/10.1177/0165551515620551

If you use the dataset in a paper, please refer https://www.kaggle.com/savasy/ttc4900 as footnote and cite one of the papers as follows:

- A Comparison of Different Approaches to Document Representation in Turkish Language, SDU Journal of Natural and Applied Science, Vol 22, Issue 2, 2018
- A comparative analysis of text classification for Turkish language, Pamukkale University Journal of Engineering Science Volume 25 Issue 5, 2018
- A Knowledge-poor Approach to Turkish Text Categorization with a Comparative Analysis, Proceedings of CICLING 2014, Springer LNCS, Nepal, 2014.
"""

_CITATION = """\
@article{doi:10.5505/pajes.2018.15931,
author = {Yıldırım, Savaş and Yıldız, Tuğba},
title = {A comparative analysis of text classification for Turkish language},
journal = {Pamukkale Univ Muh Bilim Derg},
volume = {24},
number = {5},
pages = {879-886},
year = {2018},
doi = {10.5505/pajes.2018.15931},
note ={doi: 10.5505/pajes.2018.15931},

URL = {https://dx.doi.org/10.5505/pajes.2018.15931},
eprint = {https://dx.doi.org/10.5505/pajes.2018.15931}
}
"""

_LICENSE = "CC0: Public Domain"
_HOMEPAGE = "https://www.kaggle.com/savasy/ttc4900"
_DOWNLOAD_URL = "https://raw.githubusercontent.com/savasy/TurkishTextClassification/master"
_FILENAME = "7allV03.csv"


class TTC4900Config(datasets.BuilderConfig):
    """BuilderConfig for TTC4900"""

    def __init__(self, **kwargs):
        """BuilderConfig for TTC4900.
        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(TTC4900Config, self).__init__(**kwargs)


class TTC4900(datasets.GeneratorBasedBuilder):
    """TTC4900: A Benchmark Data for Turkish Text Categorization"""

    BUILDER_CONFIGS = [
        TTC4900Config(
            name="ttc4900",
            version=datasets.Version("1.0.0"),
            description="A Benchmark Data for Turkish Text Categorization",
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "category": datasets.features.ClassLabel(
                        names=["siyaset", "dunya", "ekonomi", "kultur", "saglik", "spor", "teknoloji"]
                    ),
                    "text": datasets.Value("string"),
                }
            ),
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_HOMEPAGE,
            # License for the dataset if available
            license=_LICENSE,
            # Citation for the dataset
            citation=_CITATION,
            task_templates=[TextClassification(text_column="text", label_column="category")],
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        urls_to_download = {
            "train": _DOWNLOAD_URL + "/" + _FILENAME,
        }
        downloaded_files = dl_manager.download(urls_to_download)
        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": downloaded_files["train"]}),
        ]

    def _generate_examples(self, filepath):
        """Generate TTC4900 examples."""
        logger.info("⏳ Generating examples from = %s", filepath)
        with open(filepath, encoding="utf-8") as f:
            rdr = csv.reader(f, delimiter=",")
            next(rdr)
            rownum = 0
            for row in rdr:
                rownum += 1
                yield rownum, {
                    "category": row[0],
                    "text": row[1],
                }



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""XSum dataset."""


import json
import os

import datasets


_CITATION = """
@article{Narayan2018DontGM,
  title={Don't Give Me the Details, Just the Summary! Topic-Aware Convolutional Neural Networks for Extreme Summarization},
  author={Shashi Narayan and Shay B. Cohen and Mirella Lapata},
  journal={ArXiv},
  year={2018},
  volume={abs/1808.08745}
}
"""

_DESCRIPTION = """
Extreme Summarization (XSum) Dataset.

There are three features:
  - document: Input news article.
  - summary: One sentence summary of the article.
  - id: BBC ID of the article.

"""

# From https://github.com/EdinburghNLP/XSum/issues/12
_URL_DATA = "http://bollin.inf.ed.ac.uk/public/direct/XSUM-EMNLP18-Summary-Data-Original.tar.gz"
_URL_SPLITS = (
    "https://raw.githubusercontent.com/EdinburghNLP/XSum/master/XSum-Dataset/XSum-TRAINING-DEV-TEST-SPLIT-90-5-5.json"
)

_DOCUMENT = "document"
_SUMMARY = "summary"
_ID = "id"

_REMOVE_LINES = set(
    [
        "Share this with\n",
        "Email\n",
        "Facebook\n",
        "Messenger\n",
        "Twitter\n",
        "Pinterest\n",
        "WhatsApp\n",
        "Linkedin\n",
        "LinkedIn\n",
        "Copy this link\n",
        "These are external links and will open in a new window\n",
    ]
)


class Xsum(datasets.GeneratorBasedBuilder):
    """Extreme Summarization (XSum) Dataset."""

    # Version 1.2.0 expands coverage, includes ids, and removes web contents.
    VERSION = datasets.Version("1.2.0")

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    _DOCUMENT: datasets.Value("string"),
                    _SUMMARY: datasets.Value("string"),
                    _ID: datasets.Value("string"),
                }
            ),
            supervised_keys=(_DOCUMENT, _SUMMARY),
            homepage="https://github.com/EdinburghNLP/XSum/tree/master/XSum-Dataset",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        files_to_download = {"data": _URL_DATA, "splits": _URL_SPLITS}
        downloaded_files = dl_manager.download(files_to_download)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "split_path": downloaded_files["splits"],
                    "split_name": "train",
                    "data_dir": "bbc-summary-data",
                    "files": dl_manager.iter_archive(downloaded_files["data"]),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "split_path": downloaded_files["splits"],
                    "split_name": "validation",
                    "data_dir": "bbc-summary-data",
                    "files": dl_manager.iter_archive(downloaded_files["data"]),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "split_path": downloaded_files["splits"],
                    "split_name": "test",
                    "data_dir": "bbc-summary-data",
                    "files": dl_manager.iter_archive(downloaded_files["data"]),
                },
            ),
        ]

    def _generate_examples(self, split_path, split_name, data_dir, files):
        """Yields examples."""

        with open(split_path, "r", encoding="utf-8") as f:
            split_ids = json.load(f)
        split_ids = {k: set(v) for k, v in split_ids.items()}

        for path, f in files:
            if not split_ids[split_name]:
                break
            elif path.startswith(data_dir) and path.endswith(".summary"):
                i = os.path.basename(path).split(".")[0]
                if i in split_ids[split_name]:
                    split_ids[split_name].remove(i)
                    text = "".join(
                        [
                            line.decode("utf-8")
                            for line in f.readlines()
                            if line.decode("utf-8") not in _REMOVE_LINES and line.strip()
                        ]
                    )
                    # Each file follows below format:
                    # [SN]URL[SN]
                    # http://somelink
                    #
                    # [SN]TITLE[SN]
                    # some intro
                    #
                    # [SN]FIRST-SENTENCE[SN]
                    # some intro
                    #
                    # [SN]RESTBODY[SN]
                    # text line.
                    # another text line.
                    # "another text line."

                    # According to the following issue, FIRST-SENTENCE
                    # is the reference summary and TITLE is unused:
                    # https://github.com/EdinburghNLP/XSum/issues/22
                    segs = text.split("[SN]")
                    yield i, {_DOCUMENT: segs[8].strip(), _SUMMARY: segs[6].strip(), _ID: i}



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""TODO: Add a description here."""


import json

import datasets


# TODO: Add BibTeX citation
# Find for instance the citation on arxiv or on the dataset repo/website
_CITATION = """\
@article{roy2020lareqa,
  title={LAReQA: Language-agnostic answer retrieval from a multilingual pool},
  author={Roy, Uma and Constant, Noah and Al-Rfou, Rami and Barua, Aditya and Phillips, Aaron and Yang, Yinfei},
  journal={arXiv preprint arXiv:2004.05484},
  year={2020}
}
"""

# TODO: Add description of the dataset here
# You can copy an official description
_DESCRIPTION = """\
XQuAD-R is a retrieval version of the XQuAD dataset (a cross-lingual extractive QA dataset). Like XQuAD, XQUAD-R is an 11-way parallel dataset, where each question appears in 11 different languages and has 11 parallel correct answers across the languages.
"""

# TODO: Add a link to an official homepage for the dataset here
_HOMEPAGE = "https://github.com/google-research-datasets/lareqa"

# TODO: Add link to the official dataset URLs here
# The HuggingFace dataset library don't host the datasets but only point to the original files
# This can be an arbitrary nested dict/list of URLs (see below in `_split_generators` method)
_URL = "https://github.com/google-research-datasets/lareqa/raw/master/xquad-r/"
_LANG = ["ar", "de", "zh", "vi", "en", "es", "hi", "el", "th", "tr", "ru"]


class XquadRConfig(datasets.BuilderConfig):

    """BuilderConfig for XquadR"""

    def __init__(self, lang, **kwargs):
        """
        Args:
            lang: string, language for the input text
            **kwargs: keyword arguments forwarded to super.
        """
        super(XquadRConfig, self).__init__(version=datasets.Version("1.0.0", ""), **kwargs)
        self.lang = lang


# TODO: Name of the dataset usually match the script name with CamelCase instead of snake_case
class XquadR(datasets.GeneratorBasedBuilder):
    """TODO(xquad-r): Short description of my dataset."""

    # TODO(xquad-r): Set up version.
    VERSION = datasets.Version("1.1.0")
    BUILDER_CONFIGS = [XquadRConfig(name=f"{lang}", description=_DESCRIPTION, lang=lang) for lang in _LANG]

    def _info(self):
        # TODO(xquad-r): Specifies the datasets.DatasetInfo object
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "context": datasets.Value("string"),
                    "question": datasets.Value("string"),
                    "answers": datasets.features.Sequence(
                        {
                            "text": datasets.Value("string"),
                            "answer_start": datasets.Value("int32"),
                        }
                    ),
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO(xquad-r): Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        urls_to_download = {lang: _URL + f"{lang}.json" for lang in _LANG}
        downloaded_files = dl_manager.download_and_extract(urls_to_download)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": downloaded_files[self.config.lang]},
            ),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        # TODO(xquad-r): Yields (key, example) tuples from the dataset
        with open(filepath, encoding="utf-8") as f:
            data = json.load(f)
            for article in data["data"]:
                for paragraph in article["paragraphs"]:
                    context = paragraph["context"].strip()
                    for qa in paragraph["qas"]:
                        question = qa["question"].strip()
                        id_ = qa["id"]

                        answer_starts = [answer["answer_start"] for answer in qa["answers"]]
                        answers = [answer["text"].strip() for answer in qa["answers"]]

                        # Features currently used are "context", "question", and "answers".
                        # Others are extracted here for the ease of future expansions.
                        yield id_, {
                            "context": context,
                            "question": question,
                            "id": id_,
                            "answers": {
                                "answer_start": answer_starts,
                                "text": answers,
                            },
                        }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""TupleInf Open IE Dataset"""


import os

import datasets


_CITATION = """\
@article{Khot2017AnsweringCQ,
  title={Answering Complex Questions Using Open Information Extraction},
  author={Tushar Khot and A. Sabharwal and Peter Clark},
  journal={ArXiv},
  year={2017},
  volume={abs/1704.05572}
}
"""

_DESCRIPTION = """\
The TupleInf Open IE dataset contains Open IE tuples extracted from 263K sentences that were used by the solver \
in “Answering Complex Questions Using Open Information Extraction” (referred as Tuple KB, T). \
These sentences were collected from a large Web corpus using training questions from 4th and 8th grade as queries. \
This dataset contains 156K sentences collected for 4th grade questions and 107K sentences for 8th grade questions. \
Each sentence is followed by the Open IE v4 tuples using their simple format.
"""

_HOMEPAGE = "https://allenai.org/data/tuple-ie"

_URL = "https://ai2-public-datasets.s3.amazonaws.com/tuple-ie/TupleInfKB.zip"

_DOMAIN_FILES = {"4th_grade": "4thGradeOpenIE.txt", "8th_grade": "8thGradeOpenIE.txt"}


class TupleIEConfig(datasets.BuilderConfig):
    """BuilderConfig for TupleIE"""

    def __init__(self, *args, domains=None, **kwargs):
        super().__init__(*args, **kwargs)
        self.domains = domains


class TupleIE(datasets.GeneratorBasedBuilder):
    """TupleInf Open IE Dataset"""

    BUILDER_CONFIGS = [
        TupleIEConfig(
            name="all",
            domains=list(_DOMAIN_FILES.keys()),
            description="collected using training questions from 4th and 8th grade as queries.",
        )
    ] + [
        TupleIEConfig(
            name=name, domains=[name], description=f"collected using training questions from {name} as queries."
        )
        for name in _DOMAIN_FILES.keys()
    ]
    BUILDER_CONFIG_CLASS = TupleIEConfig
    DEFAULT_CONFIG_NAME = "all"

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "sentence": datasets.Value("string"),
                    "tuples": datasets.features.Sequence(
                        {
                            "score": datasets.Value("float"),
                            "tuple_text": datasets.Value("string"),
                            "context": datasets.Value("string"),
                            "arg1": datasets.Value("string"),
                            "rel": datasets.Value("string"),
                            "arg2s": datasets.features.Sequence(datasets.Value("string")),
                        }
                    ),
                }
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        data_dir = os.path.join(dl_manager.download_and_extract(_URL), "TupleInfKB")
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"data_dir": data_dir},
            )
        ]

    def _generate_examples(self, data_dir):
        """Yields examples."""
        id_ = -1
        for domain in self.config.domains:
            with open(os.path.join(data_dir, _DOMAIN_FILES[domain]), encoding="utf-8") as f:
                all_text = f.read()
            samples = all_text.split("\n\n")
            for sample in samples:
                rows = sample.split("\n")
                item = {"sentence": rows[0], "tuples": []}
                tuple_lines = rows[1:]
                for tuple_line in tuple_lines:
                    score, tuple_text = tuple_line.split(" ", 1)
                    context, arg1, rel, arg2s = self._decode_tuple_text(tuple_text)
                    item["tuples"].append(
                        {
                            "score": score,
                            "tuple_text": tuple_text,
                            "context": context,
                            "arg1": arg1,
                            "rel": rel,
                            "arg2s": arg2s,
                        }
                    )
                id_ += 1
                yield id_, item

    def _decode_tuple_text(self, tuple_text):
        """Decompose the tuple text into arguments and relations

        Args:
            tuple_text (str): Format of extraction text:
            .. code-block::
                {Context(<context>):}(<arg1>; <rel>; {[L|T]:}<arg2_1>; {[L|T]:}<arg2_2>; ...)

        .. note::
            * ``{}`` means one can be optionally appear
            * ``[L|T]`` means ``L`` or ``T``
            * ``L`` means spatial/location argument
            * ``T`` means temporal argument
            * We can have multiple arg2s
        """
        context = ""
        arg1 = ""
        rel = ""
        arg2s = []
        if tuple_text.startswith("Context("):
            context, tuple_text = tuple_text.split(":", 1)
            context = context[len("Context(") : -1]

        args = tuple_text[1:-1].split("; ")
        arg1, rel = args[:2]
        arg2s = args[2:]
        return context, arg1, rel, arg2s



"""TODO(event2Mind): Add a description here."""


import csv
import os

import datasets


# TODO(event2Mind): BibTeX citation
_CITATION = """\
@inproceedings{event2Mind,
    title={Event2Mind: Commonsense Inference on Events, Intents, and Reactions},
    author={Hannah Rashkin and Maarten Sap and Emily Allaway and Noah A. Smith† Yejin Choi},
    year={2018}
}
"""

# TODO(event2Mind):\

_DESCRIPTION = """\
In Event2Mind, we explore the task of understanding stereotypical intents and reactions to events. Through crowdsourcing, we create a large corpus with 25,000 events and free-form descriptions of their intents and reactions, both of the event's subject and (potentially implied) other participants.
"""
_URL = "https://uwnlp.github.io/event2mind/data/event2mind.zip"


class Event2mind(datasets.GeneratorBasedBuilder):
    """TODO(event2Mind): Short description of my dataset."""

    # TODO(event2Mind): Set up version.
    VERSION = datasets.Version("0.1.0")

    def _info(self):
        # TODO(event2Mind): Specifies the datasets.DatasetInfo object
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    # These are the features of your dataset like images, labels ...
                    "Source": datasets.Value("string"),
                    "Event": datasets.Value("string"),
                    "Xintent": datasets.Value("string"),
                    "Xemotion": datasets.Value("string"),
                    "Otheremotion": datasets.Value("string"),
                    "Xsent": datasets.Value("string"),
                    "Osent": datasets.Value("string"),
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="https://uwnlp.github.io/event2mind/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO(event2Mind): Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        dl_dir = dl_manager.download_and_extract(_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": os.path.join(dl_dir, "train.csv")},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": os.path.join(dl_dir, "test.csv")},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": os.path.join(dl_dir, "dev.csv")},
            ),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        # TODO(event2Mind): Yields (key, example) tuples from the dataset
        with open(filepath, encoding="utf-8") as f:
            data = csv.DictReader(f)
            for id_, row in enumerate(data):
                yield id_, {
                    "Source": row["Source"],
                    "Event": row["Event"],
                    "Xintent": row["Xintent"],
                    "Xemotion": row["Xemotion"],
                    "Otheremotion": row["Otheremotion"],
                    "Xsent": row["Xsent"],
                    "Osent": row["Osent"],
                }



from dataclasses import dataclass
from typing import ClassVar, Dict

from ..features import Features, Value
from .base import TaskTemplate


@dataclass(frozen=True)
class AutomaticSpeechRecognition(TaskTemplate):
    task: str = "automatic-speech-recognition"
    # TODO(lewtun): Replace input path feature with dedicated `Audio` features
    # when https://github.com/huggingface/datasets/pull/2324 is implemented
    input_schema: ClassVar[Features] = Features({"audio_file_path": Value("string")})
    label_schema: ClassVar[Features] = Features({"transcription": Value("string")})
    audio_file_path_column: str = "audio_file_path"
    transcription_column: str = "transcription"

    @property
    def column_mapping(self) -> Dict[str, str]:
        return {self.audio_file_path_column: "audio_file_path", self.transcription_column: "transcription"}



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""KorQuAD v2.1:The Korean Question Answering Dataset"""


import json
import os

import datasets


_CITATION = """\
@article{NODE09353166,
    author={Youngmin Kim,Seungyoung Lim;Hyunjeong Lee;Soyoon Park;Myungji Kim},
    title={{KorQuAD 2.0: Korean QA Dataset for Web Document Machine Comprehension}},
    booltitle={{Journal of KIISE 제47권 제6호}},
    journal={{Journal of KIISE}},
    volume={{47}},
    issue={{6}},
    publisher={The Korean Institute of Information Scientists and Engineers},
    year={2020},
    ISSN={{2383-630X}},
    pages={577-586},
    url={http://www.dbpia.co.kr/journal/articleDetail?nodeId=NODE09353166}}
"""

_DESCRIPTION = """\
KorQuAD 2.0 is a Korean question and answering dataset consisting of a total of 100,000+ pairs. There are three major differences from KorQuAD 1.0, which is the standard Korean Q & A data. The first is that a given document is a whole Wikipedia page, not just one or two paragraphs. Second, because the document also contains tables and lists, it is necessary to understand the document structured with HTML tags. Finally, the answer can be a long text covering not only word or phrase units, but paragraphs, tables, and lists. As a baseline model, BERT Multilingual is used, released by Google as an open source. It shows 46.0% F1 score, a very low score compared to 85.7% of the human F1 score. It indicates that this data is a challenging task. Additionally, we increased the performance by no-answer data augmentation. Through the distribution of this data, we intend to extend the limit of MRC that was limited to plain text to real world tasks of various lengths and formats.
"""
_HOMEPAGE = "https://korquad.github.io/"
_LICENSE = "CC BY-ND 2.0 KR"

_URL = "https://github.com/korquad/korquad.github.io/raw/master/dataset/KorQuAD_2.1"
_URLS = {
    "train": [
        _URL + "/train/KorQuAD_2.1_train_00.zip",
        _URL + "/train/KorQuAD_2.1_train_01.zip",
        _URL + "/train/KorQuAD_2.1_train_02.zip",
        _URL + "/train/KorQuAD_2.1_train_03.zip",
        _URL + "/train/KorQuAD_2.1_train_04.zip",
        _URL + "/train/KorQuAD_2.1_train_05.zip",
        _URL + "/train/KorQuAD_2.1_train_06.zip",
        _URL + "/train/KorQuAD_2.1_train_07.zip",
        _URL + "/train/KorQuAD_2.1_train_08.zip",
        _URL + "/train/KorQuAD_2.1_train_09.zip",
        _URL + "/train/KorQuAD_2.1_train_10.zip",
        _URL + "/train/KorQuAD_2.1_train_11.zip",
        _URL + "/train/KorQuAD_2.1_train_12.zip",
    ],
    "validation": [_URL + "/dev/KorQuAD_2.1_dev_00.zip", _URL + "/dev/KorQuAD_2.1_dev_01.zip"],
}


class SquadKorV2(datasets.GeneratorBasedBuilder):
    """KorQuAD 2.1 dataset"""

    VERSION = datasets.Version("2.1.0")
    BUILDER_CONFIGS = [
        datasets.BuilderConfig(name="squad_kor_v2", version=VERSION, description=_DESCRIPTION),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "context": datasets.Value("string"),
                    "question": datasets.Value("string"),
                    "answer": datasets.Features(
                        {
                            "text": datasets.Value("string"),
                            "answer_start": datasets.Value("int32"),
                            "html_answer_start": datasets.Value("int32"),
                        }
                    ),
                    "url": datasets.Value("string"),
                    "raw_html": datasets.Value("string"),
                }
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # download and extract URLs
        urls_to_download = _URLS
        downloaded_files = dl_manager.download_and_extract(urls_to_download)

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"dirs": downloaded_files["train"]}),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION, gen_kwargs={"dirs": downloaded_files["validation"]}
            ),
        ]

    def _generate_examples(self, dirs):
        """Yields examples."""

        for d in dirs:
            filepaths = sorted(os.scandir(d), key=lambda x: x.name)
            for filepath in filepaths:
                with open(filepath, encoding="utf-8") as f:
                    squad = json.load(f)
                    for example in squad["data"]:
                        title = example.get("title", "").strip()
                        url = example.get("url", "").strip()
                        raw_html = example.get("raw_html", "").strip()
                        context = example["context"].strip()
                        for qa in example["qas"]:
                            question = qa["question"].strip()
                            answer = qa["answer"]
                            id_ = qa["id"]

                            answer_start = answer["answer_start"]
                            html_answer_start = answer["html_answer_start"]
                            answer_text = answer["text"].strip()

                            yield id_, {
                                "title": title,
                                "context": context,
                                "question": question,
                                "id": id_,
                                "answer": {
                                    "answer_start": answer_start,
                                    "html_answer_start": html_answer_start,
                                    "text": answer_text,
                                },
                                "url": url,
                                "raw_html": raw_html,
                            }



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Sogou News"""


import csv
import ctypes

import datasets


csv.field_size_limit(int(ctypes.c_ulong(-1).value // 2))


_CITATION = """\
@misc{zhang2015characterlevel,
    title={Character-level Convolutional Networks for Text Classification},
    author={Xiang Zhang and Junbo Zhao and Yann LeCun},
    year={2015},
    eprint={1509.01626},
    archivePrefix={arXiv},
    primaryClass={cs.LG}
}
"""

_DESCRIPTION = """\
The Sogou News dataset is a mixture of 2,909,551 news articles from the SogouCA and SogouCS news corpora, in 5 categories.
The number of training samples selected for each class is 90,000 and testing 12,000. Note that the Chinese characters have been converted to Pinyin.
classification labels of the news are determined by their domain names in the URL. For example, the news with
URL http://sports.sohu.com is categorized as a sport class.
"""

_DATA_URL = "https://s3.amazonaws.com/fast-ai-nlp/sogou_news_csv.tgz"


class Sogou_News(datasets.GeneratorBasedBuilder):
    """Sogou News dataset"""

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "title": datasets.Value("string"),
                    "content": datasets.Value("string"),
                    "label": datasets.features.ClassLabel(
                        names=["sports", "finance", "entertainment", "automobile", "technology"]
                    ),
                }
            ),
            # No default supervised_keys (as we have to pass both premise
            # and hypothesis as input).
            supervised_keys=None,
            homepage="",  # didn't find a real homepage
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        archive = dl_manager.download(_DATA_URL)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": "sogou_news_csv/test.csv", "files": dl_manager.iter_archive(archive)},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": "sogou_news_csv/train.csv", "files": dl_manager.iter_archive(archive)},
            ),
        ]

    def _generate_examples(self, filepath, files):
        """This function returns the examples in the raw (text) form."""
        for path, f in files:
            if path == filepath:
                lines = (line.decode("utf-8") for line in f)
                data = csv.reader(lines)
                for id_, row in enumerate(data):
                    yield id_, {"title": row[1], "content": row[2], "label": int(row[0]) - 1}
                break



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Information Seeking in the Spirit of Learning: a Dataset for Conversational Curiosity"""


import json
import os

import datasets


_CITATION = """\
@inproceedings{rodriguez2020curiosity,
    title = {Information Seeking in the Spirit of Learning: a Dataset for Conversational Curiosity},
    author = {Pedro Rodriguez and Paul Crook and Seungwhan Moon and Zhiguang Wang},
    year = 2020,
    booktitle = {Empirical Methods in Natural Language Processing}
}
"""


_DESCRIPTION = """\
This dataset contains 14K dialogs (181K utterances) where users and assistants converse about geographic topics like
geopolitical entities and locations. This dataset is annotated with pre-existing user knowledge, message-level dialog
acts, grounding to Wikipedia, and user reactions to messages.
"""

_HOMEPAGE = "https://www.pedro.ai/curiosity"

_LICENSE = "https://github.com/facebookresearch/curiosity/blob/master/LICENSE"

_URL = "https://obj.umiacs.umd.edu/curiosity/"
_URLs = {
    "train": _URL + "curiosity_dialogs.train.json",
    "val": _URL + "curiosity_dialogs.val.json",
    "test": _URL + "curiosity_dialogs.test.json",
    "test_zero": _URL + "curiosity_dialogs.test_zero.json",
}


class CuriosityDialogsConfig(datasets.BuilderConfig):
    """BuilderConfig for Curiosity Dialogs dataset"""

    def __init__(self, **kwargs):
        """BuilderConfig for Curiosity Dialogs dataset.
        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(CuriosityDialogsConfig, self).__init__(**kwargs)


class CuriosityDialogs(datasets.GeneratorBasedBuilder):
    """Information Seeking in the Spirit of Learning: a Dataset for Conversational Curiosity"""

    VERSION = datasets.Version("1.1.0")

    BUILDER_CONFIGS = [
        CuriosityDialogsConfig(
            name="curiosity_dialogs",
            version=datasets.Version("1.1.0"),
            description="Curiosity Dialog: A Dataset for Conversational Curiosity",
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "messages": datasets.Sequence(
                        {
                            "message": datasets.Value("string"),
                            "liked": datasets.ClassLabel(names=["False", "True"]),
                            "sender": datasets.ClassLabel(names=["user", "assistant"]),
                            "facts": datasets.Sequence(
                                {
                                    "fid": datasets.Value("int32"),
                                    "used": datasets.ClassLabel(names=["False", "True"]),
                                    "source": datasets.ClassLabel(names=["section", "known", "random"]),
                                }
                            ),
                            "message_id": datasets.Value("string"),
                            "dialog_acts": datasets.Sequence(datasets.Value("string")),
                        }
                    ),
                    "known_entities": datasets.Sequence(datasets.Value("string")),
                    "focus_entity": datasets.Value("string"),
                    "dialog_id": datasets.Value("int32"),
                    "inferred_steps": datasets.ClassLabel(names=["False", "True"]),
                    "created_time": datasets.Value("int64"),
                    "aspects": datasets.Sequence(datasets.Value("string")),
                    "first_aspect": datasets.Value("string"),
                    "second_aspect": datasets.Value("string"),
                    "shuffle_facts": datasets.ClassLabel(names=["False", "True"]),
                    "related_entities": datasets.Sequence(datasets.Value("string")),
                    "tag": datasets.Value("string"),
                    "user_id": datasets.Value("int32"),
                    "assistant_id": datasets.Value("int32"),
                    "is_annotated": datasets.ClassLabel(names=["False", "True"]),
                    "user_dialog_rating": datasets.Value("int32"),
                    "user_other_agent_rating": datasets.Value("int32"),
                    "assistant_dialog_rating": datasets.Value("int32"),
                    "assistant_other_agent_rating": datasets.Value("int32"),
                    "reported": datasets.ClassLabel(names=["False", "True"]),
                    "annotated": datasets.ClassLabel(names=["False", "True"]),
                }
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        data_dir = dl_manager.download_and_extract(_URLs)
        return [
            datasets.SplitGenerator(
                name="train",
                gen_kwargs={
                    "filepath": os.path.join(data_dir["train"]),
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name="val",
                gen_kwargs={"filepath": os.path.join(data_dir["val"]), "split": "val"},
            ),
            datasets.SplitGenerator(
                name="test",
                gen_kwargs={
                    "filepath": os.path.join(data_dir["test"]),
                    "split": "test_zero",
                },
            ),
            datasets.SplitGenerator(
                name="test_zero",
                gen_kwargs={
                    "filepath": os.path.join(data_dir["test_zero"]),
                    "split": "test_zero",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""

        # Bool entries are converted to string entries because of PyArrow error
        with open(filepath, encoding="utf-8") as f:
            dataset = json.load(f)
            dialogs = dataset["dialogs"]

            for id_, data in enumerate(dialogs):
                messages = data["messages"]

                for message in messages:
                    message["liked"] = str(message["liked"])
                    facts = message["facts"]

                    for fact in facts:
                        fact["used"] = str(fact["used"])

                known_entities = data["known_entities"]
                focus_entity = data["focus_entity"]
                dialog_id = data["dialog_id"]
                inferred_steps = str(data["inferred_steps"])
                created_time = data["created_time"]
                aspects = data["aspects"]
                first_aspect = data["first_aspect"]
                second_aspect = data["second_aspect"]
                shuffle_facts = str(data["shuffle_facts"])
                related_entities = data["related_entities"]
                tag = data["tag"]
                user_id = data["user_id"]
                assistant_id = data["assistant_id"]
                is_annotated = str(data["is_annotated"])
                user_dialog_rating = data["user_dialog_rating"]
                user_other_agent_rating = data["user_other_agent_rating"]
                assistant_dialog_rating = data["assistant_dialog_rating"]
                assistant_other_agent_rating = data["assistant_other_agent_rating"]
                reported = str(data["reported"])
                annotated = str(data["annotated"])

                yield id_, {
                    "messages": messages,
                    "known_entities": known_entities,
                    "focus_entity": focus_entity,
                    "dialog_id": dialog_id,
                    "inferred_steps": inferred_steps,
                    "created_time": created_time,
                    "aspects": aspects,
                    "first_aspect": first_aspect,
                    "second_aspect": second_aspect,
                    "shuffle_facts": shuffle_facts,
                    "related_entities": related_entities,
                    "tag": tag,
                    "user_id": user_id,
                    "assistant_id": assistant_id,
                    "is_annotated": is_annotated,
                    "user_dialog_rating": user_dialog_rating,
                    "user_other_agent_rating": user_other_agent_rating,
                    "assistant_dialog_rating": assistant_dialog_rating,
                    "assistant_other_agent_rating": assistant_other_agent_rating,
                    "reported": reported,
                    "annotated": annotated,
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The SemEval-2010 Task 8 on Multi-way classification of semantic relations between pairs of nominals"""


import os

import datasets
from datasets.tasks import TextClassification


_CITATION = """\
@inproceedings{hendrickx-etal-2010-semeval,
    title = "{S}em{E}val-2010 Task 8: Multi-Way Classification of Semantic Relations between Pairs of Nominals",
    author = "Hendrickx, Iris  and
      Kim, Su Nam  and
      Kozareva, Zornitsa  and
      Nakov, Preslav  and
      {\'O} S{\'e}aghdha, Diarmuid  and
      Pad{\'o}, Sebastian  and
      Pennacchiotti, Marco  and
      Romano, Lorenza  and
      Szpakowicz, Stan",
    booktitle = "Proceedings of the 5th International Workshop on Semantic Evaluation",
    month = jul,
    year = "2010",
    address = "Uppsala, Sweden",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/S10-1006",
    pages = "33--38",
}
"""

_DESCRIPTION = """\
The SemEval-2010 Task 8 focuses on Multi-way classification of semantic relations between pairs of nominals.
The task was designed to compare different approaches to semantic relation classification
and to provide a standard testbed for future research.
"""

_URL = "https://github.com/JoelNiklaus/SemEval2010Task8/raw/main/SemEval2010_task8_all_data.zip"


class SemEval2010Task8(datasets.GeneratorBasedBuilder):
    """The SemEval-2010 Task 8 focuses on Multi-way classification of semantic relations between pairs of nominals.
    The task was designed to compare different approaches to semantic relation classification
    and to provide a standard testbed for future research."""

    VERSION = datasets.Version("1.0.0")

    def _info(self):
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # This defines the different columns of the dataset and their types
            features=datasets.Features(
                {
                    "sentence": datasets.Value("string"),
                    "relation": datasets.ClassLabel(
                        names=[
                            "Cause-Effect(e1,e2)",
                            "Cause-Effect(e2,e1)",
                            "Component-Whole(e1,e2)",
                            "Component-Whole(e2,e1)",
                            "Content-Container(e1,e2)",
                            "Content-Container(e2,e1)",
                            "Entity-Destination(e1,e2)",
                            "Entity-Destination(e2,e1)",
                            "Entity-Origin(e1,e2)",
                            "Entity-Origin(e2,e1)",
                            "Instrument-Agency(e1,e2)",
                            "Instrument-Agency(e2,e1)",
                            "Member-Collection(e1,e2)",
                            "Member-Collection(e2,e1)",
                            "Message-Topic(e1,e2)",
                            "Message-Topic(e2,e1)",
                            "Product-Producer(e1,e2)",
                            "Product-Producer(e2,e1)",
                            "Other",
                        ]
                    ),
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=datasets.info.SupervisedKeysData(input="sentence", output="relation"),
            # Homepage of the dataset for documentation
            homepage="https://semeval2.fbk.eu/semeval2.php?location=tasks&taskid=11",
            citation=_CITATION,
            task_templates=[TextClassification(text_column="sentence", label_column="relation")],
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        dl_dir = dl_manager.download_and_extract(_URL)
        data_dir = os.path.join(dl_dir, "SemEval2010_task8_all_data")
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "SemEval2010_task8_training/TRAIN_FILE.TXT"),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "SemEval2010_task8_testing_keys/TEST_FILE_FULL.TXT"),
                },
            ),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        with open(filepath, "r", encoding="us-ascii") as file:
            lines = file.readlines()
            num_lines_per_sample = 4

            for i in range(0, len(lines), num_lines_per_sample):
                idx = int(lines[i].split("\t")[0])
                sentence = lines[i].split("\t")[1][1:-2]  # remove " at the start and "\n at the end
                relation = lines[i + 1][:-1]  # remove \n at the end
                yield idx, {
                    "sentence": sentence,
                    "relation": relation,
                }



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import collections

import datasets


_DESCRIPTION = """\
Preprocessed Dataset from IWSLT'15 English-Vietnamese machine translation: English-Vietnamese.
"""

_CITATION = """\
@inproceedings{Luong-Manning:iwslt15,
        Address = {Da Nang, Vietnam}
        Author = {Luong, Minh-Thang  and Manning, Christopher D.},
        Booktitle = {International Workshop on Spoken Language Translation},
        Title = {Stanford Neural Machine Translation Systems for Spoken Language Domain},
        Year = {2015}}
"""

_DATA_URL = "https://nlp.stanford.edu/projects/nmt/data/iwslt15.en-vi/{}.{}"

# Tuple that describes a single pair of files with matching translations.
# language_to_file is the map from language (2 letter string: example 'en')
# to the file path in the extracted directory.
TranslateData = collections.namedtuple("TranslateData", ["url", "language_to_file"])


class MT_Eng_ViConfig(datasets.BuilderConfig):
    """BuilderConfig for MT_Eng_Vietnamese."""

    def __init__(self, language_pair=(None, None), **kwargs):
        """BuilderConfig for MT_Eng_Vi.
        Args:
            for the `datasets.features.text.TextEncoder` used for the features feature.
          language_pair: pair of languages that will be used for translation. Should
            contain 2-letter coded strings. First will be used at source and second
            as target in supervised mode. For example: ("vi", "en").
          **kwargs: keyword arguments forwarded to super.
        """

        description = ("Translation dataset from %s to %s") % (language_pair[0], language_pair[1])
        super(MT_Eng_ViConfig, self).__init__(
            description=description,
            version=datasets.Version("1.0.0"),
            **kwargs,
        )
        self.language_pair = language_pair


class MTEngVietnamese(datasets.GeneratorBasedBuilder):
    """English Vietnamese machine translation dataset from IWSLT2015."""

    BUILDER_CONFIGS = [
        MT_Eng_ViConfig(
            name="iwslt2015-vi-en",
            language_pair=("vi", "en"),
        ),
        MT_Eng_ViConfig(
            name="iwslt2015-en-vi",
            language_pair=("en", "vi"),
        ),
    ]
    BUILDER_CONFIG_CLASS = MT_Eng_ViConfig

    def _info(self):
        source, target = self.config.language_pair
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {"translation": datasets.features.Translation(languages=self.config.language_pair)}
            ),
            supervised_keys=(source, target),
            homepage="https://nlp.stanford.edu/projects/nmt/data/iwslt15.en-vi/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        source, target = self.config.language_pair

        files = {}
        for split in ("train", "dev", "test"):
            if split == "dev":
                dl_dir_src = dl_manager.download_and_extract(_DATA_URL.format("tst2012", source))
                dl_dir_tar = dl_manager.download_and_extract(_DATA_URL.format("tst2012", target))
            if split == "dev":
                dl_dir_src = dl_manager.download_and_extract(_DATA_URL.format("tst2013", source))
                dl_dir_tar = dl_manager.download_and_extract(_DATA_URL.format("tst2013", target))
            if split == "train":
                dl_dir_src = dl_manager.download_and_extract(_DATA_URL.format(split, source))
                dl_dir_tar = dl_manager.download_and_extract(_DATA_URL.format(split, target))

            files[split] = {"source_file": dl_dir_src, "target_file": dl_dir_tar}

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs=files["train"]),
            datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs=files["dev"]),
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs=files["test"]),
        ]

    def _generate_examples(self, source_file, target_file):
        """This function returns the examples in the raw (text) form."""
        with open(source_file, encoding="utf-8") as f:
            source_sentences = f.read().split("\n")
        with open(target_file, encoding="utf-8") as f:
            target_sentences = f.read().split("\n")

        source, target = self.config.language_pair
        for idx, (l1, l2) in enumerate(zip(source_sentences, target_sentences)):
            result = {"translation": {source: l1, target: l2}}
            # Make sure that both translations are non-empty.
            yield idx, result



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""LIAR is a dataset for fake news detection with annotated claims."""


import csv
import os

import datasets


_CITATION = """\
@inproceedings{wang-2017-liar,
title = "{``}Liar, Liar Pants on Fire{''}: A New Benchmark Dataset for Fake News Detection",
author = "Wang, William Yang",
booktitle = "Proceedings of the 55th Annual Meeting of the Association for Computational Linguistics (Volume 2: Short Papers)",
month = jul,
year = "2017",
address = "Vancouver, Canada",
publisher = "Association for Computational Linguistics",
url = "https://www.aclweb.org/anthology/P17-2067",
doi = "10.18653/v1/P17-2067",
pages = "422--426",
abstract = "Automatic fake news detection is a challenging problem in deception detection, and it has tremendous real-world political and social impacts. However, statistical approaches to combating fake news has been dramatically limited by the lack of labeled benchmark datasets. In this paper, we present LIAR: a new, publicly available dataset for fake news detection. We collected a decade-long, 12.8K manually labeled short statements in various contexts from PolitiFact.com, which provides detailed analysis report and links to source documents for each case. This dataset can be used for fact-checking research as well. Notably, this new dataset is an order of magnitude larger than previously largest public fake news datasets of similar type. Empirically, we investigate automatic fake news detection based on surface-level linguistic patterns. We have designed a novel, hybrid convolutional neural network to integrate meta-data with text. We show that this hybrid approach can improve a text-only deep learning model.",
}
"""

_DESCRIPTION = """\
LIAR is a dataset for fake news detection with 12.8K human labeled short statements from politifact.com's API, and each statement is evaluated by a politifact.com editor for its truthfulness. The distribution of labels in the LIAR dataset is relatively well-balanced: except for 1,050 pants-fire cases, the instances for all other labels range from 2,063 to 2,638. In each case, the labeler provides a lengthy analysis report to ground each judgment.
"""

_HOMEPAGE = "https://www.aclweb.org/anthology/P17-2067"

_LICENSE = "Unknown"

_URL = "https://www.cs.ucsb.edu/~william/data/liar_dataset.zip"


class Liar(datasets.GeneratorBasedBuilder):
    """LIAR is a dataset for fake news detection with annotated claims."""

    VERSION = datasets.Version("1.0.0")

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "label": datasets.ClassLabel(
                        names=[
                            "false",
                            "half-true",
                            "mostly-true",
                            "true",
                            "barely-true",
                            "pants-fire",
                        ]
                    ),
                    "statement": datasets.Value("string"),
                    "subject": datasets.Value("string"),
                    "speaker": datasets.Value("string"),
                    "job_title": datasets.Value("string"),
                    "state_info": datasets.Value("string"),
                    "party_affiliation": datasets.Value("string"),
                    "barely_true_counts": datasets.Value("float"),
                    "false_counts": datasets.Value("float"),
                    "half_true_counts": datasets.Value("float"),
                    "mostly_true_counts": datasets.Value("float"),
                    "pants_on_fire_counts": datasets.Value("float"),
                    "context": datasets.Value("string"),
                }
            ),
            supervised_keys=("statement", "label"),
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        data_dir = dl_manager.download_and_extract(_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "train.tsv"),
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": os.path.join(data_dir, "test.tsv"), "split": "test"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "valid.tsv"),
                    "split": "valid",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""

        with open(filepath, encoding="utf-8") as tsv_file:
            reader = csv.reader(tsv_file, delimiter="\t", quoting=csv.QUOTE_NONE)
            for id_, row in enumerate(reader):
                yield id_, {
                    "id": row[0],
                    "label": row[1],
                    "statement": row[2],
                    "subject": row[3],
                    "speaker": row[4],
                    "job_title": row[5],
                    "state_info": row[6],
                    "party_affiliation": row[7],
                    "barely_true_counts": row[8],
                    "false_counts": row[9],
                    "half_true_counts": row[10],
                    "mostly_true_counts": row[11],
                    "pants_on_fire_counts": row[12],
                    "context": row[13],
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""PIQA dataset."""


import json
import os

import datasets


_CITATION = """\
@inproceedings{Bisk2020,
  author = {Yonatan Bisk and Rowan Zellers and
            Ronan Le Bras and Jianfeng Gao
            and Yejin Choi},
  title = {PIQA: Reasoning about Physical Commonsense in
           Natural Language},
  booktitle = {Thirty-Fourth AAAI Conference on
               Artificial Intelligence},
  year = {2020},
}
"""

_DESCRIPTION = """\
To apply eyeshadow without a brush, should I use a cotton swab or a toothpick?
Questions requiring this kind of physical commonsense pose a challenge to state-of-the-art
natural language understanding systems. The PIQA dataset introduces the task of physical commonsense reasoning
and a corresponding benchmark dataset Physical Interaction: Question Answering or PIQA.

Physical commonsense knowledge is a major challenge on the road to true AI-completeness,
including robots that interact with the world and understand natural language.

PIQA focuses on everyday situations with a preference for atypical solutions.
The dataset is inspired by instructables.com, which provides users with instructions on how to build, craft,
bake, or manipulate objects using everyday materials.

The underlying task is formualted as multiple choice question answering:
given a question `q` and two possible solutions `s1`, `s2`, a model or
a human must choose the most appropriate solution, of which exactly one is correct.
The dataset is further cleaned of basic artifacts using the AFLite algorithm which is an improvement of
adversarial filtering. The dataset contains 16,000 examples for training, 2,000 for development and 3,000 for testing.
"""

_URLs = {
    "train-dev": "https://storage.googleapis.com/ai2-mosaic/public/physicaliqa/physicaliqa-train-dev.zip",
    "test": "https://yonatanbisk.com/piqa/data/tests.jsonl",
}


class Piqa(datasets.GeneratorBasedBuilder):
    """PIQA dataset."""

    VERSION = datasets.Version("1.1.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="plain_text",
            description="Plain text",
            version=VERSION,
        )
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "goal": datasets.Value("string"),
                    "sol1": datasets.Value("string"),
                    "sol2": datasets.Value("string"),
                    "label": datasets.ClassLabel(names=["0", "1"]),
                }
            ),
            supervised_keys=None,
            homepage="https://yonatanbisk.com/piqa/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        data_dir = dl_manager.download_and_extract(_URLs)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "input_filepath": os.path.join(data_dir["train-dev"], "physicaliqa-train-dev", "train.jsonl"),
                    "label_filepath": os.path.join(data_dir["train-dev"], "physicaliqa-train-dev", "train-labels.lst"),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "input_filepath": data_dir["test"],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "input_filepath": os.path.join(data_dir["train-dev"], "physicaliqa-train-dev", "dev.jsonl"),
                    "label_filepath": os.path.join(data_dir["train-dev"], "physicaliqa-train-dev", "dev-labels.lst"),
                },
            ),
        ]

    def _generate_examples(self, input_filepath, label_filepath=None):
        """Yields examples."""
        with open(input_filepath, encoding="utf-8") as input_file:
            inputs = input_file.read().splitlines()

            if label_filepath is not None:
                with open(label_filepath, encoding="utf-8") as label_file:
                    labels = label_file.read().splitlines()
            else:
                # Labels are not available for the test set.
                # Filling the `label` column with -1 by default
                labels = [-1] * len(inputs)

            for idx, (row, lab) in enumerate(zip(inputs, labels)):
                data = json.loads(row)
                goal = data["goal"]
                sol1 = data["sol1"]
                sol2 = data["sol2"]
                yield idx, {"goal": goal, "sol1": sol1, "sol2": sol2, "label": lab}
